#if 0
globe.exe:   globe.obj winmain.obj; link globe winmain ddraw.lib dinput.lib dxguid.lib user32.lib gdi32.lib winmm.lib /nologo /opt:nowin98
globe.obj:   globe.c     sysmain.h; cl /c /J /TP /MD /nologo globe.c     /Ox /Ob2 /G6Fy
winmain.obj: winmain.cpp sysmain.h; cl /c /J /TP /MD /nologo winmain.cpp /Ox /Ob2 /G6Fy /DNOSOUND
!if 0
#endif

#include "sysmain.h"
#include <math.h>
#include <fcntl.h>
#include <io.h>
#include <sys\types.h>
#include <sys\stat.h>
#include <stdio.h>
#include <stdlib.h>
#include <mmsystem.h>

static float fmousx, fmousy;
static long bstatus = 0, obstatus = 0;
static TIME_ZONE_INFORMATION tzinfo;

//Planetary data&calculations in this program came from:
//   http://spitfire.ausys.se/psr/comp/ppcomp.html
//   http://spitfire.ausys.se/psr/comp/tutorial.html

#define USEGLOBEINSIDE 1 //1=old method=wastes tons of memory=slower loading

#define MINMILESABOVESURFACE (30000/5280)
static double startlng = -71.4845; //KENSHOUSELNG in degrees
static double startlat = -41.6611; //KENSHOUSELAT in degrees
#if (USEGLOBEINSIDE != 0)
#define MAXCROSSESPERLINE 512
#else
#define MAXCROSSESPERLINE 4
#endif
#define PI 3.141592653589793
#define PI2 (PI+PI)
#define SPEEDOFLIGHT 186282.3976 //miles/sec
#define AU (149597870/1.609344) //miles
#define SUNMOONRISETANGLE (-0.833*PI/180.0)
#define DOTRISETANGLE (-0.583*PI/180.0)   //Don't sub .25 for small objects
	// 0     Center of Sun's disk touches a mathematical horizon
	//-0.25  Sun's upper limb touches a mathematical horizon
	//-0.583 Center of Sun's disk touches the horizon; atmospheric refraction accounted for
	//-0.833 Sun's upper limb touches the horizon; atmospheric refraction accounted for (international convention)
	//-6     Civil twilight (one can no longer read outside without artificial illumination)
	//-12    Nautical twilight (navigation using a sea horizon no longer possible)
	//-15    Amateur astronomical twilight (sky dark enough for most astronomical observations)
	//-18    Astronomical twilight (sky completely dark)

typedef struct { double x, y, z; } dpoint3d;
typedef struct { float x, y, z; } fpoint3d;

static char drawearth = 1, transparent = 0, polborders = 1;
static char gravity = 1, tiltfixmode = 1, texturemode;
static long *ylookup;

static char pow2char[8] = {1,2,4,8,16,32,64,128};
static long pow2long[32] =
{
	1,2,4,8,16,32,64,128,256,512,1024,2048,4096,8192,16384,32768,65536,1<<17,1<<18,
	1<<19,1<<20,1<<21,1<<22,1<<23,1<<24,1<<25,1<<26,1<<27,1<<28,1<<29,1<<30,1<<31,
};

typedef struct { long f, p, x, y; } tiletype;
static tiletype gdd;

static double zdim, halfxdim, halfydim, halfzdim, halfdimsq;
static char zoompause = 0;

static double gd, gcosearthtilt, gsinearthtilt;
static long gsecond, gminute, ghour, gday, gmonth, gyear;
static long ototalclock;

static dpoint3d ipos, istr, ihei, ifor;
static dpoint3d ivel;
static double starang = 0;
static char angmessed = 0;

static double fhlonunits;
static long lonunits;  //Number of unique longitudes possible (360 scale factor)
static long numpoints, numloops;  //loopoff[MAXLOOPS], 5 others[MAXPOINTS]
static long *loopoff = 0;
static fpoint3d *v = 0;
#if (USEGLOBEINSIDE != 0)
typedef struct { long lng, lat; } lltype;
static lltype *ll = 0;
static long *point2 = 0, *index = 0;
static long lmaxdeltlon, lmaxdatelon;
static double maxdeltlon, maxdatelon;
#endif

#define LPSALLOC 1024    //WARNING! Make this too low and crash time!
#define LPSNUM 640       //For large loops, this forces at least this many points/lps
#define LPSTOLERANCE 0.1 //Maximum circle size (except when LPSNUM forces)
#define LPSMAXDIST 0.5   //Maximum circle size (no exceptions)
typedef struct { float x, y, z, c, s; long np, nl, le; } lpstype;
static lpstype *lps = NULL;  //LooPSummarytype: works like bounding box, but on a sphere
static long lpstotal;  //Not really needed, but nice to have handy

static long numstars;
static char *starcol = NULL;
static fpoint3d *starv = NULL;

#define MAXSNOWS 4096
static fpoint3d snowpos[MAXSNOWS];

static long numpolbords;
static char *polineto = NULL;
static fpoint3d *polv = NULL;

#if (USEGLOBEINSIDE == 0)
static unsigned char *zbit, **ybit; //8 bits per pixel: 1920,000 bytes
#endif

static long *xp = NULL, **xpptr, cirx[1024];
static long *xpcnt, cirxcnt;

#define GIFBUFLEN 16384
static char gifdata[GIFBUFLEN];
static char suffix[4100], filbuffer[768], tempstack[4096];
static long prefix[4100];

static long picxdim, picydim;
static char *pic = NULL;

	//Timer handler variables
static long totalclock, lockclock, synctics;

static const double sundia = 432449*2, moondia = 1080*2, earthdia = 3964*2;
static const double plandia[10] =
	{1516*2,3761*2,1080*2,2109*2,44375*2,37495*2,15885*2,15391*2,715*2,432449*2};

static const char plancol[10] = {0x6f,0x7f,0x8f,0x9f,0xaf,0xbf,0xcf,0xdf,0xef,0xfe};
static const char planame[10][8] =
{
	"MERCURY","VENUS","MOON","MARS","JUPITER",
	"SATURN","URANUS","NEPTUNE","PLUTO","SUN"
};
static char planamode = 1;

static const char month[12][10] =
{
	"JANUARY","FEBRUARY","MARCH","APRIL","MAY","JUNE",
	"JULY","AUGUST","SEPTEMBER","OCTOBER","NOVEMBER","DECEMBER",
};

static const char dayofweek[7][10] =
	{"SUNDAY","MONDAY","TUESDAY","WEDNESDAY","THURSDAY","FRIDAY","SATURDAY"};

#ifdef __WATCOMC__

void dtol (double, long *);
#pragma aux dtol =\
	"fistp dword ptr [eax]"\
	parm [8087][eax]

void dcalccossinasm (double *, double *, long, double);
#pragma aux dcalccossinasm =\
	"fldpi"\
	"fmulp st(1), st"\
	"fadd st, st"\
	"fsincos"\
	"fld1"\
	"fsubp st(1), st"\
	"fld st(0)"\
	"fadd st, st"\
	"fldz"\
	"fld1"\
	"shl eax, 3"\
	"add edx, eax"\
	"add ecx, eax"\
	"neg eax"\
	"beg:fst qword ptr [eax+ecx]"\
	"fadd st, st(3)"\
	"fld st(2)"\
	"fmul st, st(1)"\
	"faddp st(4), st"\
	"fxch st(1)"\
	"fst qword ptr [eax+edx]"\
	"fadd st, st(4)"\
	"fld st(2)"\
	"fmul st, st(1)"\
	"faddp st(5), st"\
	"fxch st(1)"\
	"add eax, 8"\
	"jnz short beg"\
	"fucompp"\
	"fucompp"\
	"fstp st"\
	parm [ecx][edx][eax][8087] modify exact [eax ecx edx 8087] value

void dcossin (double, double *, double *);
#pragma aux dcossin =\
	"fsincos"\
	"fstp qword ptr [eax]"\
	"fstp qword ptr [ebx]"\
	parm [8087][eax][ebx]

	//eax = min(max(eax,0),ebx)
long kbound0val (long, long);
#pragma aux kbound0val =\
	"cmp eax, ebx"\
	"jbe short skipit"\
	"sar eax, 31"\
	"not eax"\
	"and eax, ebx"\
	"skipit:"\
	parm [eax][ebx] modify exact [eax] value [eax]

void clearbufbyte (long, long, long);
#pragma aux clearbufbyte =\
	"lea ecx, [edi+edi*2]"\
	"and ecx, 3"\
	"sub edx, ecx"\
	"jle short skip1"\
	"rep stosb"\
	"mov ecx, edx"\
	"and edx, 3"\
	"shr ecx, 2"\
	"rep stosd"\
	"skip1: add ecx, edx"\
	"jl short skip2"\
	"rep stosb"\
	"skip2:"\
	parm [edi][edx][eax] modify exact [ecx edx edi] value

long bitrev (long, long);
#pragma aux bitrev =\
	"xor eax, eax"\
	"beg: shr ebx, 1"\
	"adc eax, eax"\
	"dec ecx"\
	"jnz short beg"\
	parm [ebx][ecx] modify nomemory exact [eax ebx ecx] value [eax]

long ksgn (long);
#pragma aux ksgn =\
	"add ebx, ebx"\
	"sbb eax, eax"\
	"cmp eax, ebx"\
	"adc eax, 0"\
	parm nomemory [ebx] modify exact [eax ebx]

#endif
#ifdef _MSC_VER

static _inline void dtol (double d, long *a)
{
	_asm
	{
		mov eax, a
		fld qword ptr d
		fistp dword ptr [eax]
	}
}

	//ai = radians/(2) (or ratio of angle step around full circle)
	//for(i=0;i<num;i++) { costabptr[i] = cos((double)i*ai*PI*2); sintabptr[i] = sin((double)i*ai*PI*2); }
static _inline void dcalccossinasm (double *costabptr, double *sintabptr, long num, double ai)
{
		//Excellent precision: Based on FFT.C/WAVY.C/FCALCSIN.BAS; rewritten to do both cos&sin in same loop
		//Using C wouldn't be too slow, but with ASM, I can ensure all temp calculations are in 80-bit regs :)
		//double c, s, ci, si, m; long i;
		//ai *= PI*2;
		//c = 1; ci = cos(ai)-1;
		//s = 0; si = sin(ai);
		//m = ci+ci;
		//for(i=0;i<num;i++)
		//{  costabptr[i] = c; c += ci; ci += c*m;
		//   sintabptr[i] = s; s += si; si += s*m;
		//}
	_asm
	{
		fldpi                   ;
		fmul qword ptr ai       ;ai*
		fadd st, st             ;ai**2
		fsincos                 ;cos(ai*) sin(ai*)
		fld1                    ;1 cos(ai*) sin(ai*)
		fsubp st(1), st         ;ci si
		fld st(0)               ;ci ci si
		fadd st, st             ;m ci si
		fldz                    ;s m ci si
		fld1                    ;c s m ci si
		mov eax, num
		mov edx, sintabptr
		mov ecx, costabptr
		shl eax, 3
		add edx, eax
		add ecx, eax
		neg eax
 beg: fst qword ptr [eax+ecx] ;c s m ci si
		fadd st, st(3)          ;c+ci s m ci si
		fld st(2)               ;m c+ci s m ci si
		fmul st, st(1)          ;m*(c+ci) c+ci s m ci si
		faddp st(4), st         ;c+ci s m ci+m*(c+ci) si
		fxch st(1)              ;s c+ci m ci+m*(c+ci) si
		fst qword ptr [eax+edx] ;s c m ci si
		fadd st, st(4)          ;s+si c m ci si
		fld st(2)               ;m s+si c m ci si
		fmul st, st(1)          ;m*(s+si) s+si c m ci si
		faddp st(5), st         ;s+si c m ci si+m*(s+si)
		fxch st(1)              ;c s+si m ci si+m*(s+si)
		add eax, 8
		jnz short beg
		fucompp                 ;m ci si+m*(s+si)
		fucompp                 ;si+m*(s+si)
		fstp st                 ;
	}
}

static _inline void dcossin (double a, double *c, double *s)
{
	_asm
	{
		fld qword ptr a
		fsincos
		mov eax, c
		fstp qword ptr [eax]
		mov eax, s
		fstp qword ptr [eax]
	}
}

	//if (a < 0) return(0); else if (a > b) return(b); else return(a);
static _inline long kbound0val (long a, long b) //b MUST be >= 0
{
	if ((unsigned long)a <= b) return(a);
	return((~(a>>31))&b);
}

static _inline void clearbufbyte (long d, long c, long a)
{
	_asm
	{
		push edi
		mov edi, d
		mov edx, c
		mov eax, a
		lea ecx, [edi+edi*2]
		and ecx, 3
		sub edx, ecx
		jle short skip1
		rep stosb
		mov ecx, edx
		and edx, 3
		shr ecx, 2
		rep stosd
		skip1: add ecx, edx
		jl short skip2
		rep stosb
skip2:pop edi
	}
}

static _inline long bitrev (long b, long c)
{
	_asm
	{
		mov edx, b
		mov ecx, c
		xor eax, eax
 beg: shr edx, 1
		adc eax, eax
		dec ecx
		jnz short beg
	}
}

static _inline long ksgn (long d)
{
	_asm
	{
		mov edx, d
		add edx, edx
		sbb eax, eax
		cmp eax, edx
		adc eax, 0
	}
}

#endif

//--------------------------------------------------------------
#define USEFASTCOSSIN
#ifdef USEFASTCOSSIN
	//Use this code when a single cos/sin LUT would be too big for cache
	//
	//Faster dcossin
	//fastcossininit(12960000); //Number of integer angles around circle
	//while (?) fastcossin({0..12960000-1},&c,&s);
	//fastcossinfree();
static double *fastcossinclut0 = 0, *fastcossinslut0, *fastcossinclut1, *fastcossinslut1;
static long fastcossinlogn1, fastcossinmskn1, fastcossinnumangs;
static void fastcossinfree () { if (fastcossinclut0) { free(fastcossinclut0); fastcossinclut0 = 0; } }
static void fastcossininit (long numangs)
{
	double f;
	long i, n0, n1;

	fastcossinnumangs = numangs;
	fastcossinlogn1 = (long)(log((double)numangs)*(0.5/log(2.0))+0.5);
	n1 = (1<<fastcossinlogn1); fastcossinmskn1 = n1-1; n0 = (numangs-1)/n1+1;
	fastcossinclut0 = (double *)malloc((n0+n1)*2*sizeof(double));
	if (!fastcossinclut0) { MessageBox(0,"Malloc failed (fastcossin*)",prognam,MB_OK); return; }
	fastcossinslut0 = &fastcossinclut0[n0];
	fastcossinclut1 = &fastcossinslut0[n0];
	fastcossinslut1 = &fastcossinclut1[n1];

#if 0
	f = PI*2.0/((double)numangs);
	for(i=0;i<n1;i++) dcossin(((double)i)*f,&fastcossinclut1[i],&fastcossinslut1[i]);
	f *= (double)n1;
	for(i=0;i<n0;i++) dcossin(((double)i)*f,&fastcossinclut0[i],&fastcossinslut0[i]);
#else
	f = 1.0/((double)numangs); //Much faster than ^! :)
	dcalccossinasm(fastcossinclut1,fastcossinslut1,n1,f);
	dcalccossinasm(fastcossinclut0,fastcossinslut0,n0,((double)n1)*f);
#endif
}
static __forceinline void fastcossin (long a1, double *c, double *s)
{
	long a0;
	if (a1 < 0) a1 += fastcossinnumangs;
	a0 = (a1>>fastcossinlogn1); a1 &= fastcossinmskn1;
	(*c) = fastcossinclut0[a0]*fastcossinclut1[a1] - fastcossinslut0[a0]*fastcossinslut1[a1];
	(*s) = fastcossinclut0[a0]*fastcossinslut1[a1] + fastcossinslut0[a0]*fastcossinclut1[a1];
}
#endif
//--------------------------------------------------------------

static void drawstars(double);
static long globeinside(double, double, double);
static void drawconic(double, double, double, double, double, double, double, double, double, long *, long *, long);
static double tokeplerang(double, double);
static void facedir(double, double, double);

static const long font4x6[] = //256 DOS chars, from Ken's Build SMALLFNT
{
	0x000000,0x6f9f60,0x69f960,0xaffe40,0x4efe40,0x6ff6f0,0x66f6f0,0x000000,
	0xeeaee0,0x000000,0x000000,0x000000,0x000000,0x000000,0x7755c0,0x96f690,
	0x8cec80,0x26e620,0x4e4e40,0xaaa0a0,0x7dd550,0x7ca6c0,0x000ee0,0x4e4ee0,
	0x4e4440,0x444e40,0x02f200,0x04f400,0x000000,0x000000,0x000000,0x000000,
	0x000000,0x444040,0xaa0000,0xafafa0,0x6c46c0,0xa248a0,0x4a4ce0,0x240000,
	0x488840,0x422240,0x0a4a00,0x04e400,0x000224,0x00e000,0x000040,0x224480,
	0xeaaae0,0x444440,0xe2e8e0,0xe2e2e0,0xaae220,0xe8e2e0,0xe8eae0,0xe22220,
	0xeaeae0,0xeae220,0x040400,0x040480,0x248420,0x0e0e00,0x842480,0xc24040,
	0xeaece0,0x4aeaa0,0xcacac0,0x688860,0xcaaac0,0xe8c8e0,0xe8c880,0xe8aae0,
	0xaaeaa0,0xe444e0,0xe22a60,0xaacaa0,0x8888e0,0xaeeaa0,0xaeeea0,0xeaaae0,
	0xeae880,0xeaae60,0xeacaa0,0xe8e2e0,0xe44440,0xaaaae0,0xaaa440,0xaaeea0,
	0xaa4aa0,0xaae440,0xe248e0,0xc888c0,0x844220,0x622260,0x4a0000,0x0000e0,
	0x420000,0x006a60,0x88eae0,0x00e8e0,0x22eae0,0x006e60,0x24e440,0x06a62c,
	0x88eaa0,0x040440,0x040448,0x88aca0,0x444440,0x08eee0,0x00caa0,0x00eae0,
	0x00eae8,0x00eae2,0x00e880,0x0064c0,0x04e440,0x00aa60,0x00aa40,0x00eee0,
	0x00a4a0,0x00aa6c,0x00c460,0x648460,0x440440,0xc424c0,0x6c0000,0x04ae00,
	0x68886c,0xa0aa60,0x606e60,0xe06a60,0xa06a60,0xc06a60,0x046a60,0x00e8e4,
	0xe06e60,0xa06e60,0xc06e60,0x0a0440,0x0e0440,0x0c0440,0xa4aea0,0x404ea0,
	0x60ece0,0x007a70,0x7afab0,0xe0eae0,0xa0eae0,0xc0eae0,0xe0aa60,0xc0aa60,
	0xa0aa6c,0xa0eae0,0xa0aae0,0x4e8e40,0x65c4f0,0xa4ee40,0xcafab0,0x64e4c0,
	0x606a60,0x060440,0x60eae0,0x60aa60,0xc0caa0,0xe0aea0,0x6a60e0,0xeae0e0,
	0x404860,0x007400,0x00c400,0x8a4e60,0x8a6e20,0x404440,0x05a500,0x0a5a00,
	0x282828,0x5a5a5a,0xd7d7d7,0x444444,0x44c444,0x44cc44,0x66e666,0x00e666,
	0x00cc44,0x66ee66,0x666666,0x00ee66,0x66ee00,0x66e000,0x44cc00,0x00c444,
	0x447000,0x44f000,0x00f444,0x447444,0x00f000,0x44f444,0x447744,0x667666,
	0x667700,0x007766,0x66ff00,0x00ff66,0x667766,0x00ff00,0x66ff66,0x44ff00,
	0x66f000,0x00ff44,0x00f666,0x667000,0x447700,0x007744,0x007666,0x66f666,
	0x44ff44,0x44c000,0x007444,0xffffff,0x000fff,0xcccccc,0x333333,0xfff000,
	0x00dad0,0xcacac8,0xea8880,0x00f660,0xe848e0,0x007a40,0x0aac80,0x05a220,
	0xe4a4e0,0x4aea40,0x6996f0,0x646ae0,0x06f600,0x16f680,0x68c860,0x4aaaa0,
	0xe0e0e0,0x4e40e0,0x4240e0,0x4840e0,0x4a8880,0x222a40,0x40e040,0x06ec00,
	0xeae000,0x0cc000,0x00c000,0x644c40,0xcaa000,0xc4e000,0x0eee00,0x000000,
};
static void print4x6 (long ox, long y, long fcol, long bcol, const char *fmt, ...)
{
	va_list arglist;
	char st[1024], *c, *cptr;
	long i, j, k, x;

	if (!fmt) return;
	va_start(arglist,fmt);
	if (_vsnprintf((char *)&st,sizeof(st)-1,fmt,arglist)) st[sizeof(st)-1] = 0;
	va_end(arglist);

	cptr = (char *)(y*gdd.p+gdd.f);
	for(j=20;j>=0;cptr+=gdd.p,y++,j-=4)
		if ((unsigned)y < (unsigned)gdd.y)
			for(c=st,x=ox;*c;c++,x+=4)
			{
				i = (font4x6[*c]>>j);
				for(k=0;k<4;k++)
					if ((unsigned)(x+k) < (unsigned)gdd.x)
						{ if ((i<<k)&8) cptr[x+k] = fcol; else if (bcol >= 0) cptr[x+k] = bcol; }
				if ((*c) == 9)
				{
					if (bcol >= 0) for(i=1;i<3;i++) if ((unsigned)(x+i) < (unsigned)gdd.x) cptr[x+i] = bcol;
					x += 2;
				}
			}
}

	//NOTE: font is stored vertically first!
static const long font6x8[] = //256 DOS chars, from: DOSAPP.FON (tab blank)
{
	0x00000000,0x3E000000,0x3E455145,0x6F6B3E00,0x1C003E6B,0x1C3E7C3E,0x7E3C1800,0x3000183C,0x30367F36,0x7E5C1800,0x0000185C,0x00001818,0xE7E7FFFF,0x0000FFFF,0x00000000,0xDBDBC3FF,
	0x3000FFC3,0x0E364A48,0x79290600,0x60000629,0x04023F70,0x0A7E6000,0x2A003F35,0x2A1C361C,0x3E7F0000,0x0800081C,0x007F3E1C,0x7F361400,0x00001436,0x005F005F,0x7F090600,0x22007F01,
	0x2259554D,0x60606000,0x14000060,0x14B6FFB6,0x7F060400,0x10000406,0x10307F30,0x3E080800,0x0800081C,0x08083E1C,0x40407800,0x08004040,0x083E083E,0x3F3C3000,0x0300303C,0x030F3F0F,
	0x00000000,0x00000000,0x00065F06,0x00030700,0x24000307,0x247E247E,0x6A2B2400,0x63000012,0x63640813,0x56493600,0x00005020,0x00000307,0x413E0000,0x00000000,0x00003E41,0x1C3E0800,
	0x0800083E,0x08083E08,0x60E00000,0x08000000,0x08080808,0x60600000,0x20000000,0x02040810,0x49513E00,0x00003E45,0x00407F42,0x49516200,0x22004649,0x36494949,0x12141800,0x2F00107F,
	0x31494949,0x494A3C00,0x01003049,0x03050971,0x49493600,0x06003649,0x1E294949,0x6C6C0000,0x00000000,0x00006CEC,0x22140800,0x24000041,0x24242424,0x22410000,0x02000814,0x06095901,
	0x5D413E00,0x7E001E55,0x7E111111,0x49497F00,0x3E003649,0x22414141,0x41417F00,0x7F003E41,0x41494949,0x09097F00,0x3E000109,0x7A494941,0x08087F00,0x00007F08,0x00417F41,0x40403000,
	0x7F003F40,0x41221408,0x40407F00,0x7F004040,0x7F020402,0x04027F00,0x3E007F08,0x3E414141,0x09097F00,0x3E000609,0x5E215141,0x09097F00,0x26006619,0x32494949,0x7F010100,0x3F000101,
	0x3F404040,0x40201F00,0x3F001F20,0x3F403C40,0x08146300,0x07006314,0x07087008,0x45497100,0x00000043,0x0041417F,0x08040200,0x00002010,0x007F4141,0x01020400,0x80800402,0x80808080,
	0x07030000,0x20000000,0x78545454,0x44447F00,0x38003844,0x28444444,0x44443800,0x38007F44,0x08545454,0x097E0800,0x18000009,0x7CA4A4A4,0x04047F00,0x00000078,0x00407D00,0x84804000,
	0x7F00007D,0x00442810,0x7F000000,0x7C000040,0x78041804,0x04047C00,0x38000078,0x38444444,0x4444FC00,0x38003844,0xFC444444,0x44784400,0x08000804,0x20545454,0x443E0400,0x3C000024,
	0x007C2040,0x40201C00,0x3C001C20,0x3C603060,0x10106C00,0x9C00006C,0x003C60A0,0x54546400,0x0800004C,0x0041413E,0x77000000,0x00000000,0x083E4141,0x02010200,0x3C000001,0x3C262326,
	0xE1A11E00,0x3D001221,0x007D2040,0x54543800,0x20000955,0x78555555,0x54552000,0x20007855,0x78545555,0x55572000,0x1C007857,0x1422E2A2,0x55553800,0x38000855,0x08555455,0x55553800,
	0x00000854,0x00417C01,0x79020000,0x00000042,0x00407C01,0x24297000,0x78007029,0x782F252F,0x54547C00,0x34004555,0x58547C54,0x7F097E00,0x38004949,0x00394545,0x44453800,0x39000039,
	0x00384445,0x21413C00,0x3D00007D,0x007C2041,0x60A19C00,0x3D00003D,0x003D4242,0x40413C00,0x1800003D,0x00246624,0x493E4800,0x29006249,0x292A7C2A,0x16097F00,0x40001078,0x02097E88,
	0x55542000,0x00007855,0x00417D00,0x45443800,0x3C000039,0x007D2140,0x0A097A00,0x7A000071,0x00792211,0x55550800,0x4E005E55,0x004E5151,0x4D483000,0x3C002040,0x04040404,0x04040404,
	0x17001C04,0x506A4C08,0x34081700,0x0000782A,0x00307D30,0x00140800,0x14001408,0x08140008,0x11441144,0x55AA1144,0x55AA55AA,0xEEBBEEBB,0x0000EEBB,0x0000FF00,0xFF080808,0x0A0A0000,
	0x0000FF0A,0xFF00FF08,0xF8080000,0x0000F808,0xFE0A0A0A,0xFB0A0000,0x0000FF00,0xFF00FF00,0xFA0A0000,0x0000FE02,0x0F080B0A,0x0F080000,0x00000F08,0x0F0A0A0A,0x08080000,0x0000F808,
	0x0F000000,0x08080808,0x08080F08,0xF8080808,0x00000808,0x0808FF00,0x08080808,0x08080808,0x0808FF08,0xFF000000,0xFF000A0A,0x0808FF00,0x0B080F00,0xFE000A0A,0x0A0AFA02,0x0B080B0A,
	0xFA0A0A0A,0x0A0AFA02,0xFB00FF00,0x0A0A0A0A,0x0A0A0A0A,0xFB00FB0A,0x0A0A0A0A,0x0A0A0B0A,0x0F080F08,0x0A0A0808,0x0A0AFA0A,0xF808F808,0x0F000808,0x08080F08,0x0F000000,0x00000A0A,
	0x0A0AFE00,0xF808F800,0xFF080808,0x0808FF00,0xFB0A0A0A,0x08080A0A,0x00000F08,0xF8000000,0xFFFF0808,0xFFFFFFFF,0xF0F0F0F0,0xFFFFF0F0,0x000000FF,0xFF000000,0x0F0FFFFF,0x0F0F0F0F,
	0x24241800,0xFE002418,0x00344A4A,0x01017F00,0x02000003,0x027E027E,0x49556300,0x18000063,0x041C2424,0x2020FC00,0x0800001C,0x00047804,0x77550800,0x3E000855,0x003E4949,0x02724C00,
	0x22004C72,0x00305955,0x18241800,0x18001824,0x18247E24,0x2A2A1C00,0x3C00002A,0x003C0202,0x2A2A2A00,0x0000002A,0x00242E24,0x4A4A5100,0x44000044,0x00514A4A,0xFC000000,0x20000402,
	0x00003F40,0x2A080800,0x24000808,0x00122412,0x09090600,0x00000006,0x00001818,0x00080000,0x30000000,0x02023E40,0x010E0100,0x0900000E,0x00000A0D,0x3C3C3C00,0x0000003C,0x00000000,
};
static void print6x8 (long ox, long y, long fcol, long bcol, const char *fmt, ...)
{
	va_list arglist;
	char st[1024], *c, *v, *cp, *cpx;
	long i, j, ie, x;

	if (!fmt) return;
	va_start(arglist,fmt);
	if (_vsnprintf((char *)&st,sizeof(st)-1,fmt,arglist)) st[sizeof(st)-1] = 0;
	va_end(arglist);

	cp = (char *)(y*gdd.p+gdd.f);
	for(j=1;j<256;y++,cp+=gdd.p,j+=j)
		if ((unsigned)y < (unsigned)gdd.y)
			for(c=st,x=ox;*c;c++,x+=6)
			{
				v = (char *)(((long)*c)*6 + (long)font6x8); cpx = &cp[x];
				for(i=max(-x,0),ie=min(gdd.x-x,6);i<ie;i++) { if (v[i]&j) cpx[i] = fcol; else if (bcol >= 0) cpx[i] = bcol; }
				if ((*c) == 9) { if (bcol >= 0) { for(i=max(-x,6),ie=min(gdd.x-x,18);i<ie;i++) cpx[i] = bcol; } x += 2*6; }
			}
}

	//Shore Huffman loading variables:
#define LOGHUFQSIZ 11
static long hufs[2048], hmaxbit, hqstart, hfrqend[4];
static short hufi[2048], hufq[1<<LOGHUFQSIZ];
static char hufc[2048];
static void inithuffman (char *bcnt, long *bsym, long num, short *ind, long *damaxbits, short *dahufq, long *hufqstart)
{
	long i, j, k, n, r, z, zz, maxbits, nodecnt;

		//Init destroyable index table
	nodecnt = 0;
	for(z=0;z<num;z++)
		if (bcnt[z]) ind[nodecnt++] = z;

		//Sort symbol bit sizes
	for(z=1;z<nodecnt;z++)
		for(zz=0;zz<z;zz++)
			if (bcnt[ind[z]] < bcnt[ind[zz]])
				{ i = ind[z]; ind[z] = ind[zz]; ind[zz] = i; }

		//Generate symbols
	memset(dahufq,0xff,2<<LOGHUFQSIZ); i = j = 0;
	n = 0; *damaxbits = maxbits = bcnt[ind[nodecnt-1]];
	for(z=0;z<nodecnt;z++)
	{
		n += pow2long[maxbits-bcnt[ind[z]]];
		bsym[z] = n;
		if (LOGHUFQSIZ >= bcnt[ind[z]])
		{
			*hufqstart = z+1;
			i += pow2long[LOGHUFQSIZ-bcnt[ind[z]]];
			while (j < i) dahufq[bitrev(j++,LOGHUFQSIZ)] = ind[z];
		}
	}
}
#define BITBUFSIZ 16384
static char bitbuf[BITBUFSIZ+4];
static long bitpos, globedatfil;
static void initgetbits() { bitpos = ((BITBUFSIZ+4)<<3); }
	//Warning: 0 <= dabits <= 25
static long getbits (long dabits)
{
	long obitpos;

	if (bitpos >= (BITBUFSIZ<<3))
	{
		bitpos -= (BITBUFSIZ<<3);
		*((long *)&bitbuf[0]) = *((long *)&bitbuf[BITBUFSIZ]);
		read(globedatfil,&bitbuf[4],BITBUFSIZ);
	}
	obitpos = bitpos; bitpos += dabits;
	return(((*(long *)&bitbuf[obitpos>>3]) >> (obitpos&7)) & (pow2long[dabits]-1));
}
static long lookbits (long dabits)
{
	if (bitpos >= (BITBUFSIZ<<3))
	{
		bitpos -= (BITBUFSIZ<<3);
		*((long *)&bitbuf[0]) = *((long *)&bitbuf[BITBUFSIZ]);
		read(globedatfil,&bitbuf[4],BITBUFSIZ);
	}
	return(((*(long *)&bitbuf[bitpos>>3]) >> (bitpos&7)) & (pow2long[dabits]-1));
}
static void flushbits (long dabits) { bitpos += dabits; }
static void loadshore (char *dafilename)
{
	double cx, cy, cz, dx, dy, dz, r, r2, nr, t, c;
	long i, j, k, ix, iy, px, py, z;
	long hlonunits, qlonunits, log2lonunits, lpscnt, gap, *xptr;

	if ((globedatfil = open(dafilename,O_BINARY|O_RDONLY)) == -1)
		{ MessageBox(0,"Can't load shore",0,MB_OK); return; }

	read(globedatfil,&lonunits,4);  //Total number of discrete units of longitude

	fhlonunits = ((double)lonunits)*0.5;
	hlonunits = (lonunits>>1); qlonunits = (lonunits>>2);
	for(log2lonunits=0;pow2long[log2lonunits]<lonunits;log2lonunits++);

	read(globedatfil,&numloops,4);
	read(globedatfil,&numpoints,4);

	if (v) free(v);
	if (!(v = (fpoint3d *)malloc(numpoints*sizeof(fpoint3d)))) MessageBox(0,"Out of memory (v)",0,MB_OK);

	if (lps) free(lps);
	if (!(lps = (lpstype *)malloc(LPSALLOC*sizeof(lpstype)))) MessageBox(0,"Out of memory (lps)",0,MB_OK);

	if (loopoff) free(loopoff);
	if (!(loopoff = (long *)malloc((numloops+1)*sizeof(long)))) MessageBox(0,"Out of memory (loopoff)",0,MB_OK);

#if (USEGLOBEINSIDE)
	if (ll) free(ll);
	if (!(ll = (lltype *)malloc(numpoints*sizeof(lltype)))) MessageBox(0,"Out of memory (ll)",0,MB_OK);

	if (point2 != 0) free(point2);
	if (!(point2 = (long *)malloc(numpoints*sizeof(long)))) MessageBox(0,"Out of memory (point2)",0,MB_OK);

	if (index != 0) free(index);
	if (!(index = (long *)malloc(numpoints*sizeof(long)))) MessageBox(0,"Out of memory (index)",0,MB_OK);
#endif

	initgetbits();

		//Load huffman tables for both X and Y coordinates
	hfrqend[0] = getbits(5); hfrqend[1] = getbits(5);
	memset(hufc,0,sizeof(hufc));
	for(j=0;j<hfrqend[1];j++)
		for(i=0;i<hfrqend[0];i++)
			hufc[(j<<5)+i] = getbits(5);
	inithuffman(hufc,hufs,32*26,hufi,&hmaxbit,hufq,&hqstart);

#ifndef USEFASTCOSSIN
	r = PI / fhlonunits; //Convert from lonunits to radians
#else
	fastcossininit(lonunits);
#endif

	numpoints = 0;
	for(i=0;i<numloops;i++)
	{
		loopoff[i] = numpoints;

		px = getbits(log2lonunits)-hlonunits;
		py = getbits(log2lonunits-1)-qlonunits;

#ifndef USEFASTCOSSIN
		dcossin(((double)py)*r,&t,&cy);
		dcossin(((double)px)*r,&cz,&cx);
#else
		fastcossin(py,&t,&cy);
		fastcossin(px,&cz,&cx);
#endif
#if (USEGLOBEINSIDE)
		ll[numpoints].lng = px;
		ll[numpoints].lat = py;
#endif
		v[numpoints].x = cx*t; v[numpoints].y = cy; v[numpoints].z = cz*-t;
		numpoints++;

		while (1)
		{
				//Huffman get longitude exponent
			ix = hufq[lookbits(LOGHUFQSIZ)];
			if (ix < 0)
			{
				k = bitrev(lookbits(hmaxbit),hmaxbit);
				for(z=hqstart;k>=hufs[z];z++);
				ix = hufi[z];
			}
			flushbits(hufc[ix]);
			if (!ix) break;
			iy = (ix>>5); ix &= 31;

			if (ix > 0)
			{
				if (ix > 1) ix = getbits(ix-1)+pow2long[ix-1];
				if (getbits(1)) ix = -ix;
			}
			px += ix;
			if (px < -hlonunits) px += lonunits;
			if (px >= hlonunits) px -= lonunits;

			if (iy > 0)
			{
				if (iy > 1) iy = getbits(iy-1)+pow2long[iy-1];
				if (getbits(1)) iy = -iy;
			}
			py += iy;

#ifndef USEFASTCOSSIN
			dcossin(((double)py)*r,&t,&cy);
			dcossin(((double)px)*r,&cz,&cx);
#else
			fastcossin(py,&t,&cy);
			fastcossin(px,&cz,&cx);
#endif
#if (USEGLOBEINSIDE)
			ll[numpoints].lng = px;
			ll[numpoints].lat = py;
#endif
			v[numpoints].x = cx*t; v[numpoints].y = cy; v[numpoints].z = cz*-t;
			numpoints++;
		}
	}
	loopoff[i] = numpoints;
#ifdef USEFASTCOSSIN
	fastcossinfree();
#endif
	close(globedatfil);

	//------------------------------------------------------------------------
	//All data must be ready here (numpoints, ll[*].*, numloops, loopoff[*])

#if (USEGLOBEINSIDE)
	i = 0;
	for(z=0;z<numloops;z++)
	{
		for(j=loopoff[z+1]-1;i<j;i++) point2[i] = i+1;
		point2[i] = loopoff[z]; i++;
	}
	for(i=0;i<numpoints;i++) index[i] = i;

	gap = 1; while (gap < numpoints) gap = (gap<<1)+1;
	for(gap>>=1;gap>0;gap>>=1)
	{
		k = numpoints-gap;
		for(i=0;i<k;i++)
		{
			xptr = &index[i]; j = xptr[gap];
			if (ll[xptr[0]].lng > ll[j].lng)
			{
				do
				{ xptr[gap] = xptr[0]; xptr -= gap; }
				while ((xptr >= index) && (ll[xptr[0]].lng > ll[j].lng));
				xptr[gap] = j;
			}
		}
	}

	i = numpoints-1; lmaxdeltlon = lmaxdatelon = 0;
	for(z=numloops-1;z>=0;z--)
	{
		j = k = i-(loopoff[z+1]-loopoff[z])+1;
		while (i >= k)
		{
			gap = labs(ll[i].lng-ll[j].lng);
			if (gap > lmaxdeltlon)
			{
				if (gap > hlonunits)
				{
					gap = labs(gap-lonunits);
					if (gap > lmaxdatelon) lmaxdatelon = gap;
				}
				else lmaxdeltlon = gap;
			}
			j = i; i--;
		}
	}
	//32188-byte globe.dat: lmaxdeltlon=858*lonunits/(PI*16384),lmaxdatelon=119.854036*lonunits/(PI*16384)
	//70582-byte globe.dat: lmaxdeltlon=534*lonunits/(PI*16384),lmaxdatelon=664.854036*lonunits/(PI*16384)
		//(lmaxdatelon is largest longitude delta across International Date Line)
	if (lmaxdatelon < lmaxdeltlon) lmaxdatelon = lmaxdeltlon;
	maxdatelon = (double)lmaxdatelon;
	maxdeltlon = (double)lmaxdeltlon;
#endif

	lpstotal = 0; z = 0; i = 1; j = loopoff[1]; k = 0;
	cx = v[0].x; cy = v[0].y; cz = v[0].z; r = r2 = 0;
	while (1)
	{
		i++;
		if (i >= j)
		{
			z++; if (z >= numloops) break;
			j += loopoff[z+1]-loopoff[z];
		}

		dx = v[i].x-cx; dy = v[i].y-cy; dz = v[i].z-cz;
		nr = dx*dx+dy*dy+dz*dz;
		if (nr > r2)
		{
			nr = sqrt(nr); t = (nr-r)*.5; r += t;
			if (((r > LPSTOLERANCE) && (((loopoff[z+1]-loopoff[z] > LPSNUM) && (i >= k+LPSNUM)) || (i+loopoff[z+1]-loopoff[z] == j))) || (r >= LPSMAXDIST))
			{
					//Store loop's plane-summary point to table
				c = sqrt(cx*cx + cy*cy + cz*cz); t = 1.0/c;
				lps[lpstotal].c = ((1-r2)*t + c)*.5; //c&s are cos&sin of cone angle
				lps[lpstotal].s = sqrt(1-lps[lpstotal].c*lps[lpstotal].c);
				lps[lpstotal].x = cx*t; lps[lpstotal].y = cy*t; lps[lpstotal].z = cz*t;
				lps[lpstotal].np = i; lps[lpstotal].nl = z; lps[lpstotal].le = j;
				lpstotal++;
				if (lpstotal >= LPSALLOC)
					{ MessageBox(0,"Constant LPSALLOC not high enough :(",0,MB_OK); return; }

				k = i;
				cx = v[i].x; cy = v[i].y; cz = v[i].z; r = r2 = 0;
			}
			else
			{
				t /= nr; r2 = r*r;
				cx += dx*t; cy += dy*t; cz += dz*t;
			}
		}
	}
		//Store loop's plane-summary point to table
	c = sqrt(cx*cx + cy*cy + cz*cz); t = 1.0/c;
	lps[lpstotal].c = ((1-r2)*t + c)*.5; //c&s are cos&sin of cone angle
	lps[lpstotal].s = sqrt(1-lps[lpstotal].c*lps[lpstotal].c);
	lps[lpstotal].x = cx*t; lps[lpstotal].y = cy*t; lps[lpstotal].z = cz*t;
	lps[lpstotal].np = i; lps[lpstotal].nl = z; lps[lpstotal].le = j;
	lpstotal++;
}

static void loadpolbords (char *dafilename)
{
	double cx, cy, cz, t, r, fhlonunits2;
	long i, j, k, ix, iy, px, py, z;
	long lonunits2, hlonunits, qlonunits, log2lonunits;

	if ((globedatfil = open(dafilename,O_BINARY|O_RDONLY)) == -1)
		{ MessageBox(0,"Can't load bords",0,MB_OK); return; }

	read(globedatfil,&lonunits2,4);  //Total number of discrete units of longitude

	fhlonunits2 = ((double)lonunits2)*0.5;
	hlonunits = (lonunits2>>1); qlonunits = (lonunits2>>2);
	for(log2lonunits=0;pow2long[log2lonunits]<lonunits2;log2lonunits++);

	read(globedatfil,&numpolbords,4);

	if (polv) free(polv);
	if (!(polv = (fpoint3d *)malloc(numpolbords*sizeof(fpoint3d)))) MessageBox(0,"Out of memory (polv)",0,MB_OK);

	if (polineto) free(polineto);
	if (!(polineto = (char *)malloc((numpolbords+7)>>3))) MessageBox(0,"Out of memory (polineto)",0,MB_OK);

	memset(polineto,0,(numpolbords+7)>>3);

	initgetbits();

		//Load huffman tables for both X and Y coordinates
	for(z=0;z<4;z++) hfrqend[z] = getbits(5);
	memset(hufc,0,sizeof(hufc));
	for(j=0;j<hfrqend[1];j++)
		for(i=0;i<hfrqend[0];i++)
			hufc[(j<<5)+i] = getbits(5);
	for(j=0;j<hfrqend[3];j++)
		for(i=0;i<hfrqend[2];i++)
			hufc[(j<<5)+i+1024] = getbits(5);
	inithuffman(hufc,hufs,2048,hufi,&hmaxbit,hufq,&hqstart);

#ifndef USEFASTCOSSIN
	r = PI / fhlonunits2; //Convert from lonunits to radians
#else
	fastcossininit(lonunits2);
#endif

	px = 0; py = 0;
	for(i=0;i<numpolbords;i++)
	{
			//Huffman get lng & lat exponents
		ix = hufq[lookbits(LOGHUFQSIZ)];
		if (ix < 0)
		{
			k = bitrev(lookbits(hmaxbit),hmaxbit);
			for(z=hqstart;k>=hufs[z];z++);
			ix = hufi[z];
		}
		flushbits(hufc[ix]);

		if (ix&1024) polineto[i>>3] |= pow2char[i&7];

		iy = (ix>>5)&31; ix &= 31;

		if (ix > 0)
		{
			if (ix > 1) ix = getbits(ix-1)+pow2long[ix-1];
			if (getbits(1)) ix = -ix;
		}
		px += ix;
		if (px < -hlonunits) px += lonunits2;
		if (px >= hlonunits) px -= lonunits2;

		if (iy > 0)
		{
			if (iy > 1) iy = getbits(iy-1)+pow2long[iy-1];
			if (getbits(1)) iy = -iy;
		}
		py += iy;

#ifndef USEFASTCOSSIN
		dcossin(((double)py)*r,&t,&cy);
		dcossin(((double)px)*r,&cz,&cx);
#else
		fastcossin(py,&t,&cy);
		fastcossin(px,&cz,&cx);
#endif
		polv[i].x = cx*t; polv[i].y = -cy; polv[i].z = cz*-t;
	}
#ifdef USEFASTCOSSIN
	fastcossinfree();
#endif
	close(globedatfil);
}

static void facedir (double x, double y, double z)
{
	double r, ox;

	r = x*x+z*z;
	if (r > 0)
	{
		ox = 1 / sqrt(r);
		istr.x = z*ox;
		istr.y = 0;
		istr.z = -x*ox;
		r = 1 / sqrt(r+y*y);
		ifor.x = x*r;
		ifor.y = y*r;
		ifor.z = z*r;
		ihei.x = ifor.y*istr.z - ifor.z*istr.y;
		ihei.y = ifor.z*istr.x - ifor.x*istr.z;
		ihei.z = ifor.x*istr.y - ifor.y*istr.x;
	}
}

static void facecenter ()  //Same as facedown, but doesn't force North Pole at top of screen
{
	double r;

		//Make sure istr is not parallel to ifor (is this safe enough?):
	if ((ipos.y*istr.z == ipos.z*istr.y) &&
		 (ipos.z*istr.x == ipos.x*istr.z) &&
		 (ipos.x*istr.y == ipos.y*istr.x)) return;

		//Set ifor to face Earth
	r = -1 / sqrt(ipos.x*ipos.x + ipos.y*ipos.y + ipos.z*ipos.z);
	ifor.x = ipos.x*r; ifor.y = ipos.y*r; ifor.z = ipos.z*r;

		//Suck ifor out of istr
	r = ifor.x*istr.x + ifor.y*istr.y + ifor.z*istr.z;
	istr.x -= ifor.x*r; istr.y -= ifor.y*r; istr.z -= ifor.z*r;

		//Normalize istr (/0 if istr is parallel to ifor)
	r = istr.x*istr.x + istr.y*istr.y + istr.z*istr.z;
	r = 1 / sqrt(r); istr.x *= r; istr.y *= r; istr.z *= r;

		//Cross product to get ihei
	ihei.x = ifor.y*istr.z - ifor.z*istr.y;
	ihei.y = ifor.z*istr.x - ifor.x*istr.z;
	ihei.z = ifor.x*istr.y - ifor.y*istr.x;
}

static void facedown ()
{
	double r, ox;

	r = ipos.x*ipos.x+ipos.z*ipos.z;
	if (r > 0)
	{
		ox = 1 / sqrt(r);
		istr.x = -ipos.z*ox;
		istr.y = 0;
		istr.z = ipos.x*ox;
		r = -1 / sqrt(r+ipos.y*ipos.y);
		ifor.x = ipos.x*r;
		ifor.y = ipos.y*r;
		ifor.z = ipos.z*r;
		ihei.x = ifor.y*istr.z - ifor.z*istr.y;
		ihei.y = ifor.z*istr.x - ifor.x*istr.z;
		ihei.z = ifor.x*istr.y - ifor.y*istr.x;
	}
}

static void faceup ()
{
	double r, ox;

	r = ipos.x*ipos.x+ipos.z*ipos.z;
	if (r > 0)
	{
		ox = 1 / sqrt(r);
		istr.x = -ipos.z*ox;
		istr.y = 0;
		istr.z = ipos.x*ox;
		r = 1 / sqrt(r+ipos.y*ipos.y);
		ifor.x = ipos.x*r;
		ifor.y = ipos.y*r;
		ifor.z = ipos.z*r;
		ihei.x = ifor.y*istr.z - ifor.z*istr.y;
		ihei.y = ifor.z*istr.x - ifor.x*istr.z;
		ihei.z = ifor.x*istr.y - ifor.y*istr.x;
	}
}

static void facenorth ()
{
	double r, ox;

	r = ipos.x*ipos.x+ipos.z*ipos.z;
	if (r > 0)
	{
		ox = 1 / sqrt(r);
		istr.x = -ipos.z*ox;
		istr.y = 0;
		istr.z = ipos.x*ox;
		r = 1 / sqrt(r+ipos.y*ipos.y);
		ihei.x = -ipos.x*r;
		ihei.y = -ipos.y*r;
		ihei.z = -ipos.z*r;
		ifor.x = ihei.z*istr.y - ihei.y*istr.z;
		ifor.y = ihei.x*istr.z - ihei.z*istr.x;
		ifor.z = ihei.y*istr.x - ihei.x*istr.y;
	}
}

static void facesouth ()
{
	double r, ox;

	r = ipos.x*ipos.x+ipos.z*ipos.z;
	if (r > 0)
	{
		ox = 1 / sqrt(r);
		istr.x = ipos.z*ox;
		istr.y = 0;
		istr.z = -ipos.x*ox;
		r = 1 / sqrt(r+ipos.y*ipos.y);
		ihei.x = -ipos.x*r;
		ihei.y = -ipos.y*r;
		ihei.z = -ipos.z*r;
		ifor.x = ihei.z*istr.y - ihei.y*istr.z;
		ifor.y = ihei.x*istr.z - ihei.z*istr.x;
		ifor.z = ihei.y*istr.x - ihei.x*istr.y;
	}
}

static void faceeast ()
{
	double r, ox;

	r = ipos.x*ipos.x+ipos.z*ipos.z;
	if (r > 0)
	{
		ox = 1 / sqrt(r);
		ifor.x = -ipos.z*ox;
		ifor.y = 0;
		ifor.z = ipos.x*ox;
		r = 1 / sqrt(r+ipos.y*ipos.y);
		ihei.x = -ipos.x*r;
		ihei.y = -ipos.y*r;
		ihei.z = -ipos.z*r;
		istr.x = ihei.y*ifor.z - ihei.z*ifor.y;
		istr.y = ihei.z*ifor.x - ihei.x*ifor.z;
		istr.z = ihei.x*ifor.y - ihei.y*ifor.x;
	}
}

static void facewest ()
{
	double r, ox;

	r = ipos.x*ipos.x+ipos.z*ipos.z;
	if (r > 0)
	{
		ox = 1 / sqrt(r);
		ifor.x = ipos.z*ox;
		ifor.y = 0;
		ifor.z = -ipos.x*ox;
		r = 1 / sqrt(r+ipos.y*ipos.y);
		ihei.x = -ipos.x*r;
		ihei.y = -ipos.y*r;
		ihei.z = -ipos.z*r;
		istr.x = ihei.y*ifor.z - ihei.z*ifor.y;
		istr.y = ihei.z*ifor.x - ihei.x*ifor.z;
		istr.z = ihei.x*ifor.y - ihei.y*ifor.x;
	}
}

static void getdate (long d, long *year, long *month, long *day)
{
	double yday;
	long bad;

	*year = d/(365+(6+(9.0+9.5/60.0)/60.0)/24.0)+1;
	do
	{
		yday = (((*year)-1)*365) + (((*year)-1)/4) - (((*year)-1)/100) + (((*year)-1)/400);

		//multiples of 400 are leap years. All other multiples of 100 are not
		//0-99       -0     +0     -0
		//100-199    -1     +0     -1
		//200-299    -2     +0     -2
		//300-399    -3     +0     -3
		//400-499    -4     +1     -3
		//500-599    -5     +1     -4
		//600-699    -6     +1     -5
		//700-799    -7     +1     -6
		//800-899    -8     +2     -6
		//900-999    -9     +2     -7
		//1000-1099       -10    +2     -8
		//1100-1199       -11    +2     -9
		//1200-1299       -12    +3     -9
		//1300-1399       -13    +3     -10
		//1400-1499       -14    +3     -11
		//1500-1599       -15    +3     -12
		//1600-1699       -16    +4     -12
		//1700-1799       -17    +4     -13
		//1800-1899       -18    +4     -14
		//1900-1999       -19    +4     -15
		//2000-2099       -20    +5     -15
		//2100-2199       -21    +5     -16

		bad = 0;
		if (d < yday) { (*year)--; bad = 1; }
		if (((*year)&3) || ((!((*year)%100)) && ((*year)%400)))
			{ if (d >= yday+365) { (*year)++; bad = 1; } }
		else
			{ if (d >= yday+366) { (*year)++; bad = 1; } }
	} while (bad);

	*day = d-yday;

	*month = 1;
	if ((*month == 1) && (*day >= 31)) { *month = 2; (*day) -= 31; }
	if (((*year)&3) || ((!((*year)%100)) && ((*year)%400)))
		{ if ((*month == 2) && (*day >= 28)) { *month = 3; (*day) -= 28; } }
	else
		{ if ((*month == 2) && (*day >= 29)) { *month = 3; (*day) -= 29; } }
	if ((*month == 3) && (*day >= 31)) { *month = 4; (*day) -= 31; }
	if ((*month == 4) && (*day >= 30)) { *month = 5; (*day) -= 30; }
	if ((*month == 5) && (*day >= 31)) { *month = 6; (*day) -= 31; }
	if ((*month == 6) && (*day >= 30)) { *month = 7; (*day) -= 30; }
	if ((*month == 7) && (*day >= 31)) { *month = 8; (*day) -= 31; }
	if ((*month == 8) && (*day >= 31)) { *month = 9; (*day) -= 31; }
	if ((*month == 9) && (*day >= 30)) { *month = 10; (*day) -= 30; }
	if ((*month == 10) && (*day >= 31)) { *month = 11; (*day) -= 31; }
	if ((*month == 11) && (*day >= 30)) { *month = 12; (*day) -= 30; }
	(*day)++;
}

static long getday (long year, long month, long day)
{
	long d;

	d = ((year-1)*365) + ((year-1)/4) - ((year-1)/100) + ((year-1)/400);

	if (month > 1) d += 31;
	if (month > 2)
	{
		d += 29;
		if ((year&3) || ((!(year%100)) && (year%400))) d--;
	}
	if (month > 3) d += 31;
	if (month > 4) d += 30;
	if (month > 5) d += 31;
	if (month > 6) d += 30;
	if (month > 7) d += 31;
	if (month > 8) d += 31;
	if (month > 9) d += 30;
	if (month > 10) d += 31;
	if (month > 11) d += 30;
	if (month > 12) d += 31;

	return(d+(day-1));
}

static long getdayofweek (long d)
{
	return((d+1)%7);
}

	//spring head at 2AM on first Sunday in April
	//spring back at 2AM on last Sunday in October
	//Eastern Daylight Time is during summer: EDT = GMT-4
	//Eastern Standard Time is during winter: EST = GMT-5
	//Assumes the incoming hour is always Standard Time so there
	//   aren't any nonexistent or ambiguos times near 2AM
static long isdaylight (long year, long month, long day, long hour)
{
	long d;

	if ((month > 4) && (month < 10)) return(1);
	if ((month < 4) || (month > 10)) return(0);

	if (month == 4) //April
	{
		d = getdayofweek(getday(year,4,1))+6;  //d = first Sunday in April
		if (d >= 7) d -= 7;
		d = 7-d;
		if (day < d) return(0);
		if (day > d) return(1);
		return(hour >= 2);          //1:00 1:30 3:00x 3:30x 4:00x
	}

		//October
	d = 31-getdayofweek(getday(year,10,31));  //d = last Sunday in October
	if (day < d) return(1);
	if (day > d) return(0);
	return(hour < 1);         //1:00x 1:30x 1:00 1:30 2:00
}

static void initdatetime (long dastat)
{
	double w, M, dayfrac;
	long i;
	SYSTEMTIME SysTime;

	if (dastat)
	{
		long ogsecond, ogyear;
		ogsecond = ((gday*24+ghour)*60+gminute)*60+gsecond; ogyear = gyear;
		GetLocalTime(&SysTime);
		gyear = SysTime.wYear;
		gmonth = SysTime.wMonth;
		gday = SysTime.wDay;
		ghour = SysTime.wHour;
		gminute = SysTime.wMinute;
		gsecond = SysTime.wSecond;
		//? = SysTime.wMilliseconds;
		if (gyear == ogyear)
			ototalclock += (((gday*24+ghour)*60+gminute)*60+gsecond - ogsecond)*1000;
		else
			ototalclock += 1000; //hope that system clock incremented by 1 second at the year crossing
	}
	else
	{
			//Fix date&time
		if (gsecond < 0) { gsecond += 60; gminute--; }
		if (gsecond >= 60) { gsecond -= 60; gminute++; }
		if (gminute < 0) { gminute += 60; ghour--; }
		if (gminute >= 60) { gminute -= 60; ghour++; }
		if (ghour < 0) { ghour += 24; gday--; }
		if (ghour >= 24) { ghour -= 24; gday++; }
		getdate((long)getday(gyear,gmonth,gday),&gyear,&gmonth,&gday);
	}

	if (!isdaylight(gyear,gmonth,gday,ghour)) i = tzinfo.Bias+tzinfo.StandardBias;
													 else i = tzinfo.Bias+tzinfo.DaylightBias;
	dayfrac = ((double)ghour+((double)gminute+(double)gsecond/60.0+i)/60.0)/24.0;
	if (dastat) dayfrac += ((double)(totalclock-ototalclock))/(1000.0*60.0*60.0*24.0);

	gd = getday(gyear,gmonth,gday)-getday(1999L,12L,31L) + dayfrac;

	w = (282.9404 + 4.70935E-5*gd)*PI/180.0;    //Arg. of perigee
	M = (356.0470 + 0.9856002585*gd)*PI/180.0;  //Mean anomaly
	starang = dayfrac*PI*2 + w + M + PI;

	dcossin((23.4393 - 3.563E-7*gd)*(PI/180.0),&gcosearthtilt,&gsinearthtilt);

		//Precession (Earth's axis is wobbling)
	//eclipticlongitudeoffs = 3.82394E-5*(365.2422*(epoch-2000.0)-d)*PI/180.0;

	if (dastat) angmessed = 0;
}

static void initposatkenshouse ()
{
	double r;

	dcossin(startlat*PI/180.0,&r,&ipos.y);
	dcossin(startlng*PI/180.0,&ipos.z,&ipos.x);
	ipos.z *= -r; ipos.x *= r;

	facesouth();

	gravity = 1; ivel.x = ivel.y = ivel.z = 0;

	drawearth = 1; transparent = 0; polborders = 1;
	planamode = 1; tiltfixmode = 1;
	starang = 0; angmessed = 0;
	ototalclock = totalclock; initdatetime(1L);
}

static double tokeplerang (double ia, double k)
{
	double oa, ooa, dacos, dasin;
	long cnt;

		//Make sure 0 < ia < PI*2
	ia -= (double)((long)(ia/PI2))*PI2;
	while (ia < 0) ia += PI2;
	while (ia >= PI2) ia -= PI2;

	dcossin(ia,&dacos,&dasin);
	oa = ia - k*dasin * (1 - k*dacos);
	for(cnt=7;cnt>=0;cnt--) //WARNING! Doesn't work when k = -1
	{
		dcossin(oa,&dacos,&dasin);
		ooa = oa; oa += (ia-oa - k*dasin) / (1 + k*dacos);
		if (ooa == oa) return(oa);
	}
	return(oa);
}

static void getmoonpos (double *retx, double *rety, double *retz)
{
	double t, dist, ox, oy, oz;
	double N, i, w, a, e, M, E, x, y, z, dacos, dasin;
	double Ls, Lm, Ms, Mm, D, F, Sw, SM;
	double lonperturb, latperturb, radperturb;

	N = (125.1228 - 0.0529538083*gd)*PI/180.0;  //Long asc. node
	i = 5.1454*PI/180.0;         //Inclination
	w = (318.0634 + 0.1643573223*gd)*PI/180.0;  //Arg. of perigee
	a = 60.2666;            //Mean distance
	e = 0.054900;                //Eccentricity
	M = (115.3654 + 13.0649929509*gd)*PI/180.0; //Mean anomaly

	E = tokeplerang(M,-e);

	Sw = (282.9404 + 4.70935E-5*gd)*PI/180.0;   //sun's longitude of perihelion
	SM = (356.0470 + 0.9856002585*gd)*PI/180.0; //sun's mean anomaly
	Ls = Sw+SM;
	Lm = N+w+M;
	Ms = SM;
	Mm = M;
	D = Lm-Ls;
	F = Lm-N;

	//Ls -= (double)((long)(Ls/PI2))*PI2; while (Ls < 0) Ls += PI2; while (Ls >= PI2) Ls -= PI2;
	//Lm -= (double)((long)(Lm/PI2))*PI2; while (Lm < 0) Lm += PI2; while (Lm >= PI2) Lm -= PI2;
	//Ms -= (double)((long)(Ms/PI2))*PI2; while (Ms < 0) Ms += PI2; while (Ms >= PI2) Ms -= PI2;
	//Mm -= (double)((long)(Mm/PI2))*PI2; while (Mm < 0) Mm += PI2; while (Mm >= PI2) Mm -= PI2;
	//D  -= (double)((long)(D /PI2))*PI2; while (D  < 0) D  += PI2; while (D  >= PI2) D  -= PI2;
	//F  -= (double)((long)(F /PI2))*PI2; while (F  < 0) F  += PI2; while (F  >= PI2) F  -= PI2;
	//print4x6(0L,32L,31,-1,"%lf %lf %lf %lf %lf %lf\n",Ms*180/PI,Mm*180/PI,Ls*180/PI,Lm*180/PI,D*180/PI,F*180/PI);

		//Perturbations in longitude (degrees):
	lonperturb = -1.274*sin(Mm-2*D)   +0.658*sin(2*D)       -0.186*sin(Ms)
					 -0.059*sin(2*Mm-2*D) -0.057*sin(Mm-2*D+Ms) +0.053*sin(Mm+2*D)
					 +0.046*sin(2*D-Ms)   +0.041*sin(Mm-Ms)     -0.035*sin(D)
					 -0.031*sin(Mm+Ms)    -0.015*sin(2*F-2*D)   +0.011*sin(Mm-4*D);

		//Perturbations in latitude (degrees):
	latperturb = -0.173*sin(F-2*D)    -0.055*sin(Mm-F-2*D)  -0.046*sin(Mm+F-2*D)
					 +0.033*sin(F+2*D)    +0.017*sin(2*Mm+F);

		//Perturbations in lunar distance (Earth radii):
	radperturb = -0.58*cos(Mm-2*D)    -0.46*cos(2*D);

	//print4x6(0L,32L,31,-1,"%lf %lf %lf\n",lonperturb, latperturb, radperturb);
	a += radperturb;

	x = a*(cos(E)-e);
	y = a*sqrt(1-e*e)*sin(E);

	dacos = x*cos(w) - y*sin(w);
	dasin = x*sin(w) + y*cos(w);
	x = (cos(N)*dacos - sin(N)*dasin*cos(i));
	y = (sin(N)*dacos + cos(N)*dasin*cos(i));
	z = dasin * sin(i);

	dcossin(lonperturb*(PI/180.0),&dacos,&dasin);
	ox = x*dacos - y*dasin;
	oy = y*dacos + x*dasin;
	oz = z;

	dcossin(latperturb*(PI/180.0),&dacos,&dasin);
	x = ox*dacos - oz*dasin*(ox/sqrt(ox*ox+oy*oy));
	y = oy*dacos - oz*dasin*(oy/sqrt(ox*ox+oy*oy));
	z = oz*dacos + sqrt(ox*ox+oy*oy)*dasin;

	//print4x6(0L,32L,31,-1,"%lf %lf %lf\n",atan2(y,x)*(180.0/PI)+360.0,atan2(z,sqrt(x*x+y*y))*(180.0/PI),sqrt(x*x+y*y+z*z));

	//x = r * cos(RA) * cos(Decl)
	//y = r * sin(RA) * cos(Decl)
	//z = r * sin(Decl)
	//
	//r    = sqrt( x*x + y*y + z*z )
	//RA   = atan2( y, x )
	//Decl = asin( z / r ) = atan2( z, sqrt( x*x + y*y ) )

	ox = y;
	oy = -z;
	oz = -x;

	x = ox*gcosearthtilt + oy*gsinearthtilt;
	y = oy*gcosearthtilt - ox*gsinearthtilt;
	z = oz;

		//Geocentric -> Topocentric conversion (don't assume you're at center of earth)
		//Since ipos is earth radii, must do this when x,y,z are in Earth radii
	dcossin(starang,&dacos,&dasin);
	x -= (ipos.x*dacos-ipos.z*dasin);
	y -= ipos.y;
	z -= (ipos.z*dacos+ipos.x*dasin);

		//Must convert distance from Earth radii to millions of miles
	t = 0.238857 / 60.2666;
	*retx = x*t;
	*rety = y*t;
	*retz = z*t;
}

static void getplanpos (long daplan, double *retx, double *rety, double *retz)
{
	double cx, cy, cz, nx, ny, nz, r, t, dist, ox, oy, oz;
	double N, i, w, a, e, M, E, x, y, z, dacos, dasin;
	double Mj, Ms, Mu, lonperturb, latperturb, S, P, lonecl, latecl;

		//Hack for sun at center
	if (daplan == 9) { *retx = *rety = *retz = 0; return; }

	if (daplan < 8)  //Not Pluto
	{
		switch(daplan)
		{
			case 0: //Mercury
				N = (48.3313 + 3.24587E-5*gd)*PI/180.0;     //Long asc. node
				i = (7.0047 + 5.00E-8*gd)*PI/180.0;    //Inclination
				w = (29.1241 + 1.01444E-5*gd)*PI/180.0;     //Arg. of perigee
				a = 0.387098;                //Mean distance (AU)
				e = 0.205635 + 5.59E-10*gd;       //Eccentricity
				M = (168.6562 + 4.0923344368*gd)*PI/180.0;  //Mean anomaly
				break;
			case 1: //Venus:
				N =  (76.6799 + 2.46590E-5*gd)*PI/180.0;    //Long asc. node
				i = (3.3946 + 2.75E-8*gd)*PI/180.0;    //Inclination
				w =  (54.8910 + 1.38374E-5*gd)*PI/180.0;    //Arg. of perigee
				a = 0.723330;                //Mean distance (AU)
				e = 0.006773 - 1.302E-9*gd;       //Eccentricity
				M =  (48.0052 + 1.6021302244*gd)*PI/180.0;  //Mean anomaly
				break;
			case 2: //Earth:
				N = (0.0)*PI/180.0;          //Long asc. node
				i = (0.0)*PI/180.0;          //Inclination
				w = (282.9404 + 4.70935E-5*gd)*PI/180.0;    //Arg. of perigee
				a = -1.000000;               //Mean distance (AU)
				e = 0.016709 - 1.151E-9*gd;       //Eccentricity
				M = (356.0470 + 0.9856002585*gd)*PI/180.0;  //Mean anomaly
				break;
			case 3: //Mars:
				N =  (49.5574 + 2.11081E-5*gd)*PI/180.0;    //Long asc. node
				i = (1.8497 - 1.78E-8*gd)*PI/180.0;    //Inclination
				w = (286.5016 + 2.92961E-5*gd)*PI/180.0;    //Arg. of perigee
				a = 1.523688;                //Mean distance (AU)
				e = 0.093405 + 2.516E-9*gd;       //Eccentricity
				M = (18.6021 + 0.5240207766*gd)*PI/180.0;   //Mean anomaly
				break;
			case 4: //Jupiter:
				N = (100.4542 + 2.76854E-5*gd)*PI/180.0;    //Long asc. node
				i = (1.3030 - 1.557E-7*gd)*PI/180.0;   //Inclination
				w = (273.8777 + 1.64505E-5*gd)*PI/180.0;    //Arg. of perigee
				a = 5.20256;            //Mean distance (AU)
				e = 0.048498 + 4.469E-9*gd;       //Eccentricity
				M = (19.8950 + 0.0830853001*gd)*PI/180.0;   //Mean anomaly
				break;
			case 5: //Saturn:
				N = (113.6634 + 2.38980E-5*gd)*PI/180.0;    //Long asc. node
				i = (2.4886 - 1.081E-7*gd)*PI/180.0;   //Inclination
				w = (339.3939 + 2.97661E-5*gd)*PI/180.0;    //Arg. of perigee
				a = 9.55475;            //Mean distance (AU)
				e = 0.055546 - 9.499E-9*gd;       //Eccentricity
				M = (316.9670 + 0.0334442282*gd)*PI/180.0;  //Mean anomaly
				break;
			case 6: //Uranus:
				N = (74.0005 + 1.3978E-5*gd)*PI/180.0;      //Long asc. node
				i = (0.7733 + 1.9E-8*gd)*PI/180.0;     //Inclination
				w = (96.6612 + 3.0565E-5*gd)*PI/180.0;      //Arg. of perigee
				a = 19.18171 - 1.55E-8*gd;        //Mean distance (AU)
				e = 0.047318 + 7.45E-9*gd;        //Eccentricity
				M = (142.5905 + 0.011725806*gd)*PI/180.0;   //Mean anomaly
				break;
			case 7: //Neptune:
				N = (131.7806 + 3.0173E-5*gd)*PI/180.0;     //Long asc. node
				i = (1.7700 - 2.55E-7*gd)*PI/180.0;    //Inclination
				w = (272.8461 - 6.027E-6*gd)*PI/180.0;      //Arg. of perigee
				a = 30.05826 + 3.313E-8*gd;       //Mean distance (AU)
				e = 0.008606 + 2.15E-9*gd;        //Eccentricity
				M = (260.2471 + 0.005995147*gd)*PI/180.0;   //Mean anomaly
				break;
		}

		E = tokeplerang(M,-e);

		//a += radperturb;  //Warning: a is in AU units at this point

		x = a*(cos(E)-e);
		y = a*sqrt(1-e*e)*sin(E);

		dacos = x*cos(w) - y*sin(w);
		dasin = x*sin(w) + y*cos(w);
		x = (cos(N)*dacos - sin(N)*dasin*cos(i));
		y = (sin(N)*dacos + cos(N)*dasin*cos(i));
		z = dasin * sin(i);

		if ((daplan == 4) || (daplan == 5) || (daplan == 6))  //Perturbations
		{
			Mj = (19.8950 + 0.0830853001*gd)*PI/180.0;   //Mean anomaly of Jupiter
			Ms = (316.9670 + 0.0334442282*gd)*PI/180.0;  //Mean anomaly of Saturn
			Mu = (142.5905 + 0.011725806*gd)*PI/180.0;   //Mean anomaly of Uranus
			if (daplan == 4) //Perturbations for Jupiter. Add these terms to the longitude:
			{
				lonperturb = -0.332 * sin(2*Mj - 5*Ms - 67.6*PI/180.0)
								 -0.056 * sin(2*Mj - 2*Ms + 21*PI/180.0)
								 +0.042 * sin(3*Mj - 5*Ms + 21*PI/180.0)
								 -0.036 * sin(Mj - 2*Ms)
								 +0.022 * cos(Mj - Ms)
								 +0.023 * sin(2*Mj - 3*Ms + 52*PI/180.0)
								 -0.016 * sin(Mj - 5*Ms - 69*PI/180.0);
				latperturb = 0;
			}
			if (daplan == 5) //Perturbations for Saturn. Add these terms to the longitude:
			{
				lonperturb = +0.812 * sin(2*Mj - 5*Ms - 67.6*PI/180.0)
								 -0.229 * cos(2*Mj - 4*Ms - 2*PI/180.0)
								 +0.119 * sin(Mj - 2*Ms - 3*PI/180.0)
								 +0.046 * sin(2*Mj - 6*Ms - 69*PI/180.0)
								 +0.014 * sin(Mj - 3*Ms + 32*PI/180.0);

				latperturb = -0.020 * cos(2*Mj - 4*Ms - 2*PI/180.0)
								 +0.018 * sin(2*Mj - 6*Ms - 49*PI/180.0);
			}
			if (daplan == 6)  //Perturbations for Uranus: Add these terms to the longitude:
			{
				lonperturb = +0.040 * sin(Ms - 2*Mu + 6*PI/180.0)
								 +0.035 * sin(Ms - 3*Mu + 33*PI/180.0)
								 -0.015 * sin(Mj - Mu + 20*PI/180.0);
				latperturb = 0;
			}

			dcossin(lonperturb*(PI/180.0),&dacos,&dasin);
			ox = x*dacos - y*dasin;
			oy = y*dacos + x*dasin;
			oz = z;

			dcossin(latperturb*(PI/180.0),&dacos,&dasin);
			x = ox*dacos - oz*dasin*(ox/sqrt(ox*ox+oy*oy));
			y = oy*dacos - oz*dasin*(oy/sqrt(ox*ox+oy*oy));
			z = oz*dacos + sqrt(ox*ox+oy*oy)*dasin;
		}
	}
	else  //Special case for Pluto:
	{
		S = (50.03 + 0.033459652*gd)*(PI/180.0);
		P = (238.95 + 0.003968789*gd)*(PI/180.0);

			//heliocentric ecliptic longitude (degrees)
		lonecl = 238.9508 +0.00400703*gd
					-19.799*sin(P)   +19.848*cos(P)
					 +0.897*sin(2*P) -4.956*cos(2*P)
					 +0.610*sin(3*P) +1.211*cos(3*P)
					 -0.341*sin(4*P) -0.190*cos(4*P)
					 +0.128*sin(5*P) -0.034*cos(5*P)
					 -0.038*sin(6*P) +0.031*cos(6*P)
					 +0.020*sin(S-P) -0.010*cos(S-P);

			//heliocentric ecliptic latitude (degrees)
		latecl =  -3.9082
					 -5.453*sin(P)   -14.975*cos(P)
					 +3.527*sin(2*P) +1.673*cos(2*P)
					 -1.051*sin(3*P) +0.328*cos(3*P)
					 +0.179*sin(4*P) -0.292*cos(4*P)
					 +0.019*sin(5*P) +0.100*cos(5*P)
					 -0.031*sin(6*P) -0.026*cos(6*P)
										  +0.011*cos(S-P);

			//distance (a.u.):
		r = 40.72 +6.68*sin(P)    +6.90*cos(P)
					 -1.18*sin(2*P)  -0.03*cos(2*P)
					 +0.15*sin(3*P)  -0.14*cos(3*P);

		x = r*cos(lonecl*PI/180.0)*cos(latecl*PI/180.0);
		y = r*sin(lonecl*PI/180.0)*cos(latecl*PI/180.0);
		z = r*sin(latecl*PI/180.0);
	}

	//print4x6(0L,32L,31,-1,"%lf %lf %lf\n",atan2(y,x)*(180.0/PI)+360.0,atan2(z,sqrt(x*x+y*y))*(180.0/PI),sqrt(x*x+y*y+z*z));

	//x = r * cos(RA) * cos(Decl)
	//y = r * sin(RA) * cos(Decl)
	//z = r * sin(Decl)
	//
	//r    = sqrt( x*x + y*y + z*z )
	//RA   = atan2( y, x )
	//Decl = asin( z / r ) = atan2( z, sqrt( x*x + y*y ) )

		//Must convert distance from AU to millions of miles
	t = AU*.000001;
	cx = y*t;
	cy = -z*t;
	cz = -x*t;

	nx = cx*gcosearthtilt + cy*gsinearthtilt;
	ny = cy*gcosearthtilt - cx*gsinearthtilt;
	nz = cz;

	if (daplan == 2)
	{
			//Hack to account for your ipos.? position relative to Earth
			//Note: ipos.? is in earth radii, nx,ny,nz are in millions of miles
		dcossin(starang,&dacos,&dasin);
		r = earthdia*0.5*.000001;   //earth radii to millions of miles
		nx += (ipos.x*dacos-ipos.z*dasin)*r;
		ny += ipos.y*r;
		nz += (ipos.z*dacos+ipos.x*dasin)*r;

			//Hack to make Earth wobble as a result of the Moon orbiting around it
			//.0123 is mass of Moon relative to Earth
		getmoonpos(&ox,&oy,&oz);
		nx -= .0123*ox; ny -= .0123*oy; nz -= .0123*oz;
	}

	*retx = nx;
	*rety = ny;
	*retz = nz;
}

//#define DBGNEARLIN
#ifdef DBGNEARLIN
static fpoint3d dbgnearlin[3];
#endif

static void drawland ()
{
	double ox, oy, oz, nx1, ny1, nz1, nx2, ny2, nz2, f, ff, t1, t2;
	double sx1, sy1, sx2, sy2, nxoff, nyoff, nzoff, c1, c2, pnx, pny, pnz, fsc, fss;
	long i, j, k, x, y, z, xinc, x1, y1, x2, y2, maxdist, gap, *xptr;
	long cirxindex, starter, reuse, xorval;
	fpoint3d *ovp, *vp, *vpend, *nextvp, *lvp;
	unsigned char ch, *rptr, *wptr;

	fsc = 1.0/sqrt(ipos.x*ipos.x + ipos.y*ipos.y + ipos.z*ipos.z); fss = sqrt(1-fsc*fsc);
	pnx = ipos.x*fsc; pny = ipos.y*fsc; pnz = ipos.z*fsc;

	nxoff = -ipos.x*istr.x - ipos.y*istr.y - ipos.z*istr.z;
	nyoff = -ipos.x*ihei.x - ipos.y*ihei.y - ipos.z*ihei.z;
	nzoff = -ipos.x*ifor.x - ipos.y*ifor.y - ipos.z*ifor.z;

	cirxcnt = 0;
#if (USEGLOBEINSIDE == 0)
	memset(zbit,0,(gdd.x|1)*gdd.y);
#endif

	vp = lvp = v; k = 0;
	for(z=0;z<numloops;z++)
	{
		ovp = vpend = &vp[loopoff[z+1]-loopoff[z]-1];
		reuse = 0;
		for(;vp<=vpend;ovp=vp++)
		{
			if (reuse)
			{
				c1 = c2 = -1;  //Must set both because used later for cirx[]

				if (!transparent)  //Clip to horizon plane
				{
					t1 = t2; t2 = vp->x*ipos.x + vp->y*ipos.y + vp->z*ipos.z - 1.0;
					if (((long *)&t2)[1] < 0) c2 = t2/(t2-t1);
					if (vp == lvp) lvp = &v[lps[k++].np];
				}

				nz1 = nz2; nz2 = vp->x*ifor.x + vp->y*ifor.y + vp->z*ifor.z + nzoff;
				ny1 = ny2; ny2 = vp->x*ihei.x + vp->y*ihei.y + vp->z*ihei.z + nyoff;
				nx1 = nx2; nx2 = vp->x*istr.x + vp->y*istr.y + vp->z*istr.z + nxoff;
				if (((long *)&nz2)[1] < 0x3f400000)  //Clip to close front plane
				{
					f = (nz2-(1.0/2048.0))/(nz2-nz1);
					if (c2 < f) c2 = f;
				}

				if (((long *)&c2)[1] > 0)
				{
					nx2 += (nx1-nx2)*c2; ny2 += (ny1-ny2)*c2; nz2 += (nz1-nz2)*c2;
					reuse = 0;
				}
				sx1 = sx2;
				sy1 = sy2;
			}
			else
			{
				c1 = c2 = -1;  //if (reuse) { c1 = -1; & doesn't need checking! }

					//Clip to horizon plane
				if (!transparent)
				{
skipstuff:;
					t1 = ovp->x*ipos.x + ovp->y*ipos.y + ovp->z*ipos.z - 1.0;
					t2 = vp->x*ipos.x + vp->y*ipos.y + vp->z*ipos.z - 1.0;
					if (((long *)&t1)[1] < 0)
					{
						if (((long *)&t2)[1] < 0)
						{
							if (vp == lvp) //All set to check for a SUPER skip!
							{
								f = lps[k].x*pnx + lps[k].y*pny + lps[k].z*pnz - lps[k].c*fsc + lps[k].s*fss;
								lvp = &v[lps[k].np];
								if (((long *)&f)[1] < 0)
								{
									z = lps[k].nl; if (z >= numloops) break;
									vp = lvp;
									vpend = &v[lps[k].le-1];
									ovp = vp-1;
									if (lps[k].np+loopoff[z+1]-loopoff[z] == lps[k].le) ovp = vpend;
									k++;
									goto skipstuff;
								}
								k++;
							}
							continue;
						}
						c1 = t1/(t1-t2);
					}
					else if (((long *)&t2)[1] < 0)
						c2 = t2/(t2-t1);
					if (vp == lvp) lvp = &v[lps[k++].np];
				}

					//Note: 1/2048 in double is: 0x3f40000000000000

					//Clip to close front plane
				nz1 = ovp->x*ifor.x + ovp->y*ifor.y + ovp->z*ifor.z + nzoff;
				nz2 = vp->x*ifor.x + vp->y*ifor.y + vp->z*ifor.z + nzoff;
				if (((long *)&nz1)[1] < 0x3f400000)
				{
					if (((long *)&nz2)[1] < 0x3f400000) continue;
					f = (nz1-(1.0/2048.0))/(nz1-nz2);
					if (c1 < f) c1 = f;
					if (c1+c2 >= 1) continue; //Prevent invalid backward lines - when clipping twice!
				}
				else if (((long *)&nz2)[1] < 0x3f400000)
				{
					f = (nz2-(1.0/2048.0))/(nz2-nz1);
					if (c2 < f) c2 = f;
					if (c1+c2 >= 1) continue; //Prevent invalid backward lines - when clipping twice!
				}

				nx1 = ovp->x*istr.x + ovp->y*istr.y + ovp->z*istr.z + nxoff;
				ny1 = ovp->x*ihei.x + ovp->y*ihei.y + ovp->z*ihei.z + nyoff;
				nx2 = vp->x*istr.x + vp->y*istr.y + vp->z*istr.z + nxoff;
				ny2 = vp->x*ihei.x + vp->y*ihei.y + vp->z*ihei.z + nyoff;
				if (((long *)&c1)[1] > 0)
				{
					f = halfzdim/(nz1+(nz2-nz1)*c1);
					sx1 = (nx1+(nx2-nx1)*c1)*f + halfxdim;
					sy1 = (ny1+(ny2-ny1)*c1)*f + halfydim;
				}
				else
				{
					f = halfzdim/nz1;
					sx1 = nx1*f + halfxdim;
					sy1 = ny1*f + halfydim;
				}
				if (((long *)&c2)[1] > 0)
				{
					nx2 += (nx1-nx2)*c2; ny2 += (ny1-ny2)*c2; nz2 += (nz1-nz2)*c2;
				} else { reuse = 1; }
			}
			ff = halfzdim/nz2;
			sx2 = nx2*ff + halfxdim;
			sy2 = ny2*ff + halfydim;

			if (((long *)&sy1)[1] == ((long *)&sy2)[1])   //(sy1 == sy2)
			{
				dtol(sy1,&y);
				if ((((long *)&c1)[1] > 0) && ((unsigned)y < (unsigned)gdd.y) && (sx1+sx1 < xpptr[y][0]+xpptr[y][1])) cirx[cirxcnt++] = y;
				if ((((long *)&c2)[1] > 0) && ((unsigned)y < (unsigned)gdd.y) && (sx2+sx2 < xpptr[y][0]+xpptr[y][1])) cirx[cirxcnt++] = y;
			}
			else
			{
				if (sy1 < sy2)
				{
					if (((long *)&sy1)[1] < 0) y1 = 0; else dtol(sy1,&y1);
					if (sy2 >= gdd.y) y2 = gdd.y; else dtol(sy2,&y2);
					if ((((long *)&c1)[1] > 0) && ((unsigned)y1 < (unsigned)gdd.y) && (sx1+sx1 < xpptr[y1][0]+xpptr[y1][1])) cirx[cirxcnt++] = y1;
					if ((((long *)&c2)[1] > 0) && ((unsigned)y2 < (unsigned)gdd.y) && (sx2+sx2 < xpptr[y2][0]+xpptr[y2][1])) cirx[cirxcnt++] = y2;
					xorval = 0;
				}
				else
				{
					if (((long *)&sy2)[1] < 0) y1 = 0; else dtol(sy2,&y1);
					if (sy1 >= gdd.y) y2 = gdd.y; else dtol(sy1,&y2);
					if ((((long *)&c1)[1] > 0) && ((unsigned)y2 < (unsigned)gdd.y) && (sx1+sx1 < xpptr[y2][0]+xpptr[y2][1])) cirx[cirxcnt++] = y2;
					if ((((long *)&c2)[1] > 0) && ((unsigned)y1 < (unsigned)gdd.y) && (sx2+sx2 < xpptr[y1][0]+xpptr[y1][1])) cirx[cirxcnt++] = y1;
					xorval = 1;
				}
				if ((fabs(sx1) < 4096) && (fabs(sx2) < 4096))
				{
					ff = (sx2-sx1)*262144/(sy2-sy1); dtol(ff,&xinc);
					f = sx1*262144+((double)y1+(double).5-sy1)*ff; dtol(f,&x);
					for(;y1<y2;y1++)
					{
#if (USEGLOBEINSIDE == 0)
							/*     LEN PF ANS
									  00  0  x0
									  00  0  x0
							0       01  0  01
							1       01  1  11
							01      10  1  x0
							10      10  1  x0
							010     11  1  01
							101     11  0  11
							0101    00  0  x0
							1010    00  0  x0
							01010   01  0  01
							10101   01  1  11
							010101  10  1  x0
							101010  10  1  x0   ANS(1) = PF^(LEN>>1)
							*/
						i = (x>>18);
						if (i < gdd.x)
						{
							if (i < xpptr[y1][0]) i = xpptr[y1][0];
							ybit[y1][i] = (ybit[y1][i]+2)^xorval;
						}
#else
						xpptr[y1][xpcnt[y1]++] = (x>>18);
#endif
						x += xinc;
					}
				}
				else if (y1 != 0x80000000)
				{
					ff = (sx2-sx1)/(sy2-sy1);
					f = sx1+((double)y1+(double).5-sy1)*ff;
					for(;y1<y2;y1++)
					{
#if (USEGLOBEINSIDE == 0)
						dtol(f,&i);
						if (i < gdd.x)
						{
							if (i < xpptr[y1][0]) i = xpptr[y1][0];
							ybit[y1][i] = (ybit[y1][i]+2)^xorval;
						}
#else
						dtol(f,&xpptr[y1][xpcnt[y1]++]);
#endif
						f += ff;
					}
				}
			}
		}
	}

	gap = 1; while (gap < cirxcnt) gap = (gap<<1)+1;
	for(gap>>=1;gap>0;gap>>=1)
	{
		k = cirxcnt-gap;
		for(i=0;i<k;i++)
		{
			xptr = &cirx[i]; j = xptr[gap];
			if (xptr[0] <= j) continue;
			do
			{ xptr[gap] = xptr[0]; xptr -= gap; }
			while ((xptr >= cirx) && (xptr[0] > j));
			xptr[gap] = j;
		}
	}

	if (transparent)
		starter = 1;
	else
	{
#if (USEGLOBEINSIDE != 0)
		starter = -1;
		y1 = 0; y2 = gdd.y;
		while ((y1 < y2) && (xpcnt[y1] < 2)) y1++;
		while ((y1 < y2) && (xpcnt[y2-1] < 2)) y2--;
		y1++; y2--;  //Don't let test point near dangerously small top or bottom
		for(j=16;j;j--)
		{
			y = ((rand()*(y2-y1))>>15)+y1;
			if ((unsigned)y >= (unsigned)gdd.y) continue; //04/24/2009: prevent crash
			if (xpcnt[y] < 2) continue;
			x1 = kbound0val(xpptr[y][0],gdd.x);
			x2 = kbound0val(xpptr[y][1],gdd.x);
			x = ((rand()*(x2-x1))>>15)+x1; if ((x < x1+2) || (x >= x2-2)) continue;

			nx1 = (x+1-halfxdim)*istr.x + (y+.5-halfydim)*ihei.x + halfzdim*ifor.x;
			ny1 = (x+1-halfxdim)*istr.y + (y+.5-halfydim)*ihei.y + halfzdim*ifor.y;
			nz1 = (x+1-halfxdim)*istr.z + (y+.5-halfydim)*ihei.z + halfzdim*ifor.z;
			//(ipos.x-nx1*f)^2+(ipos.y-ny1*f)^2+(ipos.z-nz1*f)^2 = 1
			ox = nx1*nx1+ny1*ny1+nz1*nz1;
			oy = ipos.x*nx1+ipos.y*ny1+ipos.z*nz1;
			oz = ipos.x*ipos.x+ipos.y*ipos.y+ipos.z*ipos.z-1.0;
			f = (sqrt(oy*oy-ox*oz)+oy)/ox;
			starter = globeinside(ipos.x-nx1*f,ipos.y-ny1*f,ipos.z-nz1*f)^1;
			for(i=0;i<cirxcnt;i++) { if (cirx[i] > y) break; starter ^= 1; }
			for(i=xpcnt[y]-1;i>=2;i--) if (xpptr[y][i] <= x) starter ^= 1;
			break;
		}
#else
		starter = 1; cirxindex = 0;
		for(y=0;y<gdd.y;y++)
		{
			while ((cirxindex < cirxcnt) && (cirx[cirxindex] <= y))
				{ starter ^= 1; cirxindex++; }
			if (xpcnt[y] < 2) continue;
			x1 = kbound0val(xpptr[y][0],gdd.x);
			x2 = kbound0val(xpptr[y][1],gdd.x);
			rptr = ybit[y];
			for(x=x1;x<x2;x++)
				if (rptr[x]&2) break;
			if (rptr[x]&2)
			{
				if ((rptr[x]+1)&4) starter ^= 1;
				break;
			}
		}
		if (y >= gdd.y) starter = -1;
#endif
	}

#if 0
	if (keystatus[0x45]) //NumLock hack 4 testing globeinside (use lo-res!)
	{
		oz = ipos.x*ipos.x+ipos.y*ipos.y+ipos.z*ipos.z-1.0;
		for(y=0;y<gdd.y;y++)
		{
			if (xpcnt[y] < 2) continue;
			for(x=max(xpptr[y][0],0);x<min(xpptr[y][1],gdd.x);x++)
			{
				nx1 = (x+1-halfxdim)*istr.x + (y+.5-halfydim)*ihei.x + halfzdim*ifor.x;
				ny1 = (x+1-halfxdim)*istr.y + (y+.5-halfydim)*ihei.y + halfzdim*ifor.y;
				nz1 = (x+1-halfxdim)*istr.z + (y+.5-halfydim)*ihei.z + halfzdim*ifor.z;
				////(ipos.x-nx1*f)^2+(ipos.y-ny1*f)^2+(ipos.z-nz1*f)^2 = 1
				ox = nx1*nx1+ny1*ny1+nz1*nz1;
				oy = ipos.x*nx1+ipos.y*ny1+ipos.z*nz1;
				f = (sqrt(oy*oy-ox*oz)+oy)/ox;
				starter = globeinside(ipos.x-nx1*f,ipos.y-ny1*f,ipos.z-nz1*f)^1;
				*(char *)(gdd.f+ylookup[y]+x) = 0x48+(starter<<4);
			}
		}
		return;
	}
#endif

	cirxindex = 0;
	for(y=0;y<gdd.y;y++)
	{
		while ((cirxindex < cirxcnt) && (cirx[cirxindex] <= y))
			{ starter ^= 1; cirxindex++; }

		if (xpcnt[y] < 2)
			{ clearbufbyte(gdd.f+ylookup[y],gdd.x,0L); continue; }

		if (starter < 0)
		{
			nx1 = (xpptr[y][0]+1-halfxdim)*istr.x + (y+.5-halfydim)*ihei.x + halfzdim*ifor.x;
			ny1 = (xpptr[y][0]+1-halfxdim)*istr.y + (y+.5-halfydim)*ihei.y + halfzdim*ifor.y;
			nz1 = (xpptr[y][0]+1-halfxdim)*istr.z + (y+.5-halfydim)*ihei.z + halfzdim*ifor.z;
			//(ipos.x-nx1*f)^2+(ipos.y-ny1*f)^2+(ipos.z-nz1*f)^2 = 1
			ox = nx1*nx1+ny1*ny1+nz1*nz1;
			oy = ipos.x*nx1+ipos.y*ny1+ipos.z*nz1;
			oz = ipos.x*ipos.x+ipos.y*ipos.y+ipos.z*ipos.z-1.0;
			f = (sqrt(oy*oy-ox*oz)+oy)/ox;
			starter = globeinside(ipos.x-nx1*f,ipos.y-ny1*f,ipos.z-nz1*f)^1;
		}

#if (USEGLOBEINSIDE != 0)
		if (xpcnt[y] == 2)
		{
			x1 = kbound0val(xpptr[y][0],gdd.x);
			x2 = kbound0val(xpptr[y][1],gdd.x);
			clearbufbyte(gdd.f+ylookup[y],x1,0L);
			if (!starter) clearbufbyte(gdd.f+ylookup[y]+x1,x2-x1,0x48484848);
						else clearbufbyte(gdd.f+ylookup[y]+x1,x2-x1,0x58585858);
			clearbufbyte(gdd.f+ylookup[y]+x2,gdd.x-x2,0L);
			continue;
		}

		xptr = xpptr[y];
		k = xptr[xpcnt[y]-1]; xptr[xpcnt[y]-1] = xptr[1]; xptr[1] = k;

		//if (xpcnt[y] < 8)  //Use bubble sort for smaller counts
		//{
		//   for(i=2;i<xpcnt[y]-1;i++)
		//      for(j=1;j<i;j++)
		//         if (xptr[i] < xptr[j]) { k = xptr[i]; xptr[i] = xptr[j]; xptr[j] = k; }
		//}
		//else
		//{
				//Standard shell sort, but ignore first & last element (already sorted)
			gap = 1; while (gap < xpcnt[y]-2) gap = (gap<<1)+1;
			for(gap>>=1;gap>0;gap>>=1)
			{
				k = xpcnt[y]-1-gap;
				for(i=1;i<k;i++)
				{
					xptr = &xpptr[y][i]; j = xptr[gap];
					if (xptr[0] <= j) continue;
					do
					{ xptr[gap] = xptr[0]; xptr -= gap; }
					while ((xptr > xpptr[y]) && (xptr[0] > j));
					xptr[gap] = j;
				}
			}
		//}
		xptr = xpptr[y];

		if (starter&1) j = 0x58585858; else j = 0x48484848;
		clearbufbyte(gdd.f+ylookup[y],kbound0val(xptr[0],gdd.x),0L);
		for(i=0;i<xpcnt[y];i++)
		{
			x1 = kbound0val(xptr[i],gdd.x);
			clearbufbyte(gdd.f+ylookup[y]+x1,kbound0val(xptr[i+1],gdd.x)-x1,j);
			j ^= 0x10101010;
		}
		x2 = kbound0val(xptr[xpcnt[y]-1],gdd.x);
		clearbufbyte(gdd.f+ylookup[y]+x2,gdd.x-x2,0L);
#else
		x1 = kbound0val(xpptr[y][0],gdd.x);
		x2 = kbound0val(xpptr[y][1],gdd.x);
		wptr = (unsigned char *)(ylookup[y]+gdd.f);

		clearbufbyte((long)wptr,x1,0L);
		if (starter&1) ch = 0x58; else ch = 0x48;

		rptr = ybit[y];

#if 0
		if (!transparent) //THIS CASE IS ONLY NECESSARY FOR TESTING LAND/WATER LOOPS!
		{
			for(x=x1;x<x2;x++)
			{
				if (rptr[x]&2) { if ((rptr[x]+1)&4) ch = 0x58; else ch = 0x48; }
				wptr[x] = ch;
			}
		}
		else
#endif
		{
			for(x=x1;x<x2;x++)
			{
				ch ^= ((rptr[x]&2)<<3); //if (rptr[x]&2) ch ^= (0x58^0x48);
				wptr[x] = ch;
			}
		}

		clearbufbyte((long)&wptr[x2],gdd.x-x2,0L);

#endif
	}

	if (keystatus[0x3a])  //Caps Lock
	{
			//Uncomment this code to see how cool the loop summary tables are!
			//-Draws a circle around every group of points you see & it's fast too!
		double a, b, c, d, e, g, cxcx, cycy, czcz, rkx, rky, rkz, nx, ny, nz;
		double cx, cy, cz, kx, ky, kz, r;
		long col, ox0, ox1;

		double dot, bestd;
		long bestk;
		bestd = -1e32; bestk = 0;
		for(k=lpstotal-1;k>=0;k--)
		{
			dot = lps[k].x*pnx + lps[k].y*pny + lps[k].z*pnz;
			d = lps[k].c*dot - lps[k].s*sqrt(1-dot*dot);
			if (d > bestd) { bestd = d; bestk = k; }
		}
			//Refine max dist to check using quick approximation
		if (!bestk) i = 0; else i = lps[bestk-1].np;
		for(;i<lps[bestk].np;i+=16)
		{
			d = v[i].x*pnx + v[i].y*pny + v[i].z*pnz;
			if (d > bestd) bestd = d;
		}

			//Draw globeinside check radius (dark blue circle)
		r = sqrt(1-bestd*bestd);
		kx = pnx*istr.x + pny*istr.y + pnz*istr.z; cx = kx*bestd+nxoff;
		ky = pnx*ihei.x + pny*ihei.y + pnz*ihei.z; cy = ky*bestd+nyoff;
		kz = pnx*ifor.x + pny*ifor.y + pnz*ifor.z; cz = kz*bestd+nzoff;
			//3D Circle projection, Input: radius:r, center:cx,cy,cz, unit_normal:kx,ky,kz  (39 *'s)
		cxcx = cx*cx; cycy = cy*cy; czcz = cz*cz;
		rkx = r*kx; rky = r*ky; rkz = r*kz;
		nx = cy*kz - cz*ky; ny = cz*kx - cx*kz; nz = cx*ky - cy*kx;
		a = nx*nx + rkx*rkx - cycy - czcz;
		b = nx*ny + rkx*rky + cx*cy; b += b;
		c = ny*ny + rky*rky - cxcx - czcz;
		d = (nx*nz + rkx*rkz + cx*cz)*halfzdim - a*halfxdim; d += d - b*halfydim;
		e = (ny*nz + rky*rkz + cy*cz)*halfzdim - c*halfydim; e += e - b*halfxdim;
		f = (nz*nz + rkz*rkz - cxcx - cycy)*halfzdim*halfzdim - a*halfxdim*halfxdim - b*halfxdim*halfydim - c*halfydim*halfydim - d*halfxdim - e*halfydim;
		drawconic(a,b,c,d,e,f,-cx,-cy,-cz,&y1,&y2,0);
		j = ylookup[y1]+gdd.f;
		ox0 = ((xpptr[y1][0]+xpptr[y1][1])>>1); ox1 = ox0-1;
		for(y=y1;y<y2;y++)
		{
			do { ox0 += ksgn(xpptr[y][0]-ox0); *(char *)(ox0+j) = 223; } while (ox0 != xpptr[y][0]);
			do { ox1 += ksgn(xpptr[y][1]-ox1); *(char *)(ox1+j) = 223; } while (ox1 != xpptr[y][1]);
			j += ylookup[1];
		}
		if (y < gdd.y) { while (ox0 < ox1-1) { ox0++; *(char *)(ox0+j) = 223; } }


		for(k=lpstotal-1;k>=0;k--)
		{
			f = lps[k].x*pnx + lps[k].y*pny + lps[k].z*pnz - lps[k].c*fsc + lps[k].s*fss;
			if (((long *)&f)[1] < 0) continue;

			dot = lps[k].x*pnx + lps[k].y*pny + lps[k].z*pnz;
			d = lps[k].c*dot + lps[k].s*sqrt(1-dot*dot);
			if ((dot < lps[k].c) && (d < bestd)) col = 254; //yellow
													  else col = 63; //white (lps is in range of globeinside check)


			kx = lps[k].x*istr.x + lps[k].y*istr.y + lps[k].z*istr.z; cx = kx*lps[k].c+nxoff;
			ky = lps[k].x*ihei.x + lps[k].y*ihei.y + lps[k].z*ihei.z; cy = ky*lps[k].c+nyoff;
			kz = lps[k].x*ifor.x + lps[k].y*ifor.y + lps[k].z*ifor.z; cz = kz*lps[k].c+nzoff;
			r = lps[k].s;

				//3D Circle projection, Input: radius:r, center:cx,cy,cz, unit_normal:kx,ky,kz  (39 *'s)
			cxcx = cx*cx; cycy = cy*cy; czcz = cz*cz;
			rkx = r*kx; rky = r*ky; rkz = r*kz;
			nx = cy*kz - cz*ky; ny = cz*kx - cx*kz; nz = cx*ky - cy*kx;
			a = nx*nx + rkx*rkx - cycy - czcz;
			b = nx*ny + rkx*rky + cx*cy; b += b;
			c = ny*ny + rky*rky - cxcx - czcz;
			d = (nx*nz + rkx*rkz + cx*cz)*halfzdim - a*halfxdim; d += d - b*halfydim;
			e = (ny*nz + rky*rkz + cy*cz)*halfzdim - c*halfydim; e += e - b*halfxdim;
			f = (nz*nz + rkz*rkz - cxcx - cycy)*halfzdim*halfzdim - a*halfxdim*halfxdim - b*halfxdim*halfydim - c*halfydim*halfydim - d*halfxdim - e*halfydim;
			drawconic(a,b,c,d,e,f,-cx,-cy,-cz,&y1,&y2,0);
			j = ylookup[y1]+gdd.f;
			ox0 = ((xpptr[y1][0]+xpptr[y1][1])>>1); ox1 = ox0-1;
			for(y=y1;y<y2;y++)
			{
				do { ox0 += ksgn(xpptr[y][0]-ox0); *(char *)(ox0+j) = col; } while (ox0 != xpptr[y][0]);
				do { ox1 += ksgn(xpptr[y][1]-ox1); *(char *)(ox1+j) = col; } while (ox1 != xpptr[y][1]);
				j += ylookup[1];
			}
			if (y < gdd.y) { while (ox0 < ox1-1) { ox0++; *(char *)(ox0+j) = col; } }
		}
	}

#ifdef DBGNEARLIN
	f = 1.0/sqrt(ipos.x*ipos.x + ipos.y*ipos.y + ipos.z*ipos.z);
	globeinside(ipos.x*f,ipos.y*f,ipos.z*f);
	for(j=0;j<3;j++)
	{
		ox = (dbgnearlin[j].x-ipos.x)*istr.x + (dbgnearlin[j].y-ipos.y)*istr.y + (dbgnearlin[j].z-ipos.z)*istr.z;
		oy = (dbgnearlin[j].x-ipos.x)*ihei.x + (dbgnearlin[j].y-ipos.y)*ihei.y + (dbgnearlin[j].z-ipos.z)*ihei.z;
		oz = (dbgnearlin[j].x-ipos.x)*ifor.x + (dbgnearlin[j].y-ipos.y)*ifor.y + (dbgnearlin[j].z-ipos.z)*ifor.z;

		oz = halfzdim/oz;
		ox = ox*oz + halfxdim;
		oy = oy*oz + halfydim;
		for(i=0;i<64;i++)
		{
			x = ox+((float)(rand()-16384))/8192.0;
			y = oy+((float)(rand()-16384))/8192.0;
			if (((unsigned long)x < gdd.x) && ((unsigned long)y < gdd.y))
				*(char *)(gdd.f+ylookup[y]+x) = 63+64*j;
		}
	}

		//Draw point on globe that you're flying over
	ox = (ipos.x*f-ipos.x)*istr.x + (ipos.y*f-ipos.y)*istr.y + (ipos.z*f-ipos.z)*istr.z;
	oy = (ipos.x*f-ipos.x)*ihei.x + (ipos.y*f-ipos.y)*ihei.y + (ipos.z*f-ipos.z)*ihei.z;
	oz = (ipos.x*f-ipos.x)*ifor.x + (ipos.y*f-ipos.y)*ifor.y + (ipos.z*f-ipos.z)*ifor.z;
	oz = halfzdim/oz;
	ox = ox*oz + halfxdim;
	oy = oy*oz + halfydim;
	for(i=0;i<64;i++)
	{
		x = ox+((float)(rand()-16384))/4096.0;
		y = oy+((float)(rand()-16384))/4096.0;
		if (((unsigned long)x < gdd.x) && ((unsigned long)y < gdd.y))
			*(char *)(gdd.f+ylookup[y]+x) = 191;
	}
#endif
}

static void drawpolbords ()
{
	double ox, oy, oz, nx1, ny1, nz1, nx2, ny2, nz2, f, c1, c2, t1, t2;
	double nxoff, nyoff, nzoff, sx1, sy1, sx2, sy2;
	long i, inc, end, sx, sy, reuse;
	unsigned long xdimup18, ydimup18;
	fpoint3d *vp, *vpend;

	nxoff = -ipos.x*istr.x - ipos.y*istr.y - ipos.z*istr.z;
	nyoff = -ipos.x*ihei.x - ipos.y*ihei.y - ipos.z*ihei.z;
	nzoff = -ipos.x*ifor.x - ipos.y*ifor.y - ipos.z*ifor.z;

	xdimup18 = (gdd.x<<18); ydimup18 = (gdd.y<<18);

	vpend = &polv[numpolbords];
	for(vp=polv,i=0;vp<vpend;vp++,i++)
	{
		if (polineto[i>>3]&pow2char[i&7]) { reuse = 0; continue; }

		if (reuse)
		{
			c2 = -1;  //Don't need to use c1 here, but be CAREFUL when copying code!

			if (!transparent)  //Clip to horizon plane
			{
				t1 = t2; t2 = vp->x*ipos.x + vp->y*ipos.y + vp->z*ipos.z - 1.0;
				if (((long *)&t2)[1] < 0) c2 = t2/(t2-t1);
			}

			nz1 = nz2; nz2 = vp->x*ifor.x + vp->y*ifor.y + vp->z*ifor.z + nzoff;
			ny1 = ny2; ny2 = vp->x*ihei.x + vp->y*ihei.y + vp->z*ihei.z + nyoff;
			nx1 = nx2; nx2 = vp->x*istr.x + vp->y*istr.y + vp->z*istr.z + nxoff;
			if (((long *)&nz2)[1] < 0x3f400000)  //Clip to close front plane
			{
				f = (nz2-(1.0/2048.0))/(nz2-nz1);
				if (c2 < f) c2 = f;
			}

			if (((long *)&c2)[1] > 0)
			{
				nx2 += (nx1-nx2)*c2; ny2 += (ny1-ny2)*c2; nz2 += (nz1-nz2)*c2;
				reuse = 0;
			}
			sx1 = sx2;
			sy1 = sy2;
		}
		else
		{
			c1 = c2 = -1;

				//Clip to horizon plane
			if (!transparent)
			{
				t1 = vp[-1].x*ipos.x + vp[-1].y*ipos.y + vp[-1].z*ipos.z - 1.0;
				t2 = vp->x*ipos.x + vp->y*ipos.y + vp->z*ipos.z - 1.0;
				if (((long *)&t1)[1] < 0)
				{
					if (((long *)&t2)[1] < 0) continue;
					c1 = t1/(t1-t2);
				}
				else if (((long *)&t2)[1] < 0)
					c2 = t2/(t2-t1);
			}

				//Note: 1/2048 in double is: 0x3f40000000000000

				//Clip to close front plane
			nz1 = vp[-1].x*ifor.x + vp[-1].y*ifor.y + vp[-1].z*ifor.z + nzoff;
			nz2 = vp->x*ifor.x + vp->y*ifor.y + vp->z*ifor.z + nzoff;
			if (((long *)&nz1)[1] < 0x3f400000)
			{
				if (((long *)&nz2)[1] < 0x3f400000) continue;
				f = (nz1-(1.0/2048.0))/(nz1-nz2);
				if (c1 < f) c1 = f;
				if (c1+c2 >= 1) continue; //Prevent invalid backward lines - when clipping twice!
			}
			else if (((long *)&nz2)[1] < 0x3f400000)
			{
				f = (nz2-(1.0/2048.0))/(nz2-nz1);
				if (c2 < f) c2 = f;
				if (c1+c2 >= 1) continue; //Prevent invalid backward lines - when clipping twice!
			}

			nx1 = vp[-1].x*istr.x + vp[-1].y*istr.y + vp[-1].z*istr.z + nxoff;
			ny1 = vp[-1].x*ihei.x + vp[-1].y*ihei.y + vp[-1].z*ihei.z + nyoff;
			nx2 = vp->x*istr.x + vp->y*istr.y + vp->z*istr.z + nxoff;
			ny2 = vp->x*ihei.x + vp->y*ihei.y + vp->z*ihei.z + nyoff;
			if (((long *)&c1)[1] > 0)
			{
				f = halfzdim/(nz1+(nz2-nz1)*c1);
				sx1 = (nx1+(nx2-nx1)*c1)*f + halfxdim;
				sy1 = (ny1+(ny2-ny1)*c1)*f + halfydim;
			}
			else
			{
				f = halfzdim/nz1;
				sx1 = nx1*f + halfxdim;
				sy1 = ny1*f + halfydim;
			}
			if (((long *)&c2)[1] > 0)
			{
				nx2 += (nx1-nx2)*c2; ny2 += (ny1-ny2)*c2; nz2 += (nz1-nz2)*c2;
			}
			else reuse = 1;
		}
		f = halfzdim/nz2;
		sx2 = nx2*f + halfxdim;
		sy2 = ny2*f + halfydim;

		if (fabs(sx2-sx1) >= fabs(sy2-sy1))
		{
			if (sx2 > sx1) { dtol(sx1,&sx); dtol(sx2,&end); }
						 else { dtol(sx2,&sx); dtol(sx1,&end); }
			if (sx < 0) sx = 0;
			if (end > gdd.x) end = gdd.x;
			f = (sy2-sy1)/(sx2-sx1); dtol(f*262144,&inc);
			dtol((((double)sx+(double)0.5-sx1)*f+sy1)*262144,&sy);
			for(;sx<end;sx++,sy+=inc)
				if ((unsigned)sy < ydimup18)
					*(char *)(ylookup[sy>>18]+sx+gdd.f) = 69;
		}
		else
		{
			if (sy2 > sy1) { dtol(sy1,&sy); dtol(sy2,&end); }
						 else { dtol(sy2,&sy); dtol(sy1,&end); }
			if (sy < 0) sy = 0;
			if (end > gdd.y) end = gdd.y;
			f = (sx2-sx1)/(sy2-sy1); dtol(f*262144,&inc);
			dtol((((double)sy+(double)0.5-sy1)*f+sx1)*262144,&sx);
			for(;sy<end;sy++,sx+=inc)
				if ((unsigned)sx < xdimup18)
					*(char *)(ylookup[sy]+(sx>>18)+gdd.f) = 69;
		}
	}
}

static void drawstars (double dastarang)
{
	fpoint3d *vp;
	dpoint3d nipos, nistr, nihei, nifor;
	double fsx, fsy, nx, ny, nz, ox, oy, oz, nzmin, t, cosang, sinang;
	double a, b, c, d, e, f, g, ax, ay, az, bx, by, bz, cx, cy, cz;
	double px[10], py[10], pz[10], oa, cxcx, cycy, czcz, r;
	double nsq, kx, ky, kz, rkx, rky, rkz, ux, uy, uz, tx, ty, tof, r2;
	long i, j, k, sx, sy, sz, x, y, z, x1, x2, col[4], pi[10];
	long miny, maxy, nminy, nmaxy, usecol, morecol;
	char *scr;

	nipos = ipos; nistr = istr; nihei = ihei; nifor = ifor;
	dcossin(dastarang,&cosang,&sinang);
	ox = nipos.x; oy = nipos.z;
	nipos.x = ox*cosang-oy*sinang;
	nipos.z = oy*cosang+ox*sinang;
	ox = nistr.x; oy = nistr.z;
	nistr.x = ox*cosang-oy*sinang;
	nistr.z = oy*cosang+ox*sinang;
	ox = nihei.x; oy = nihei.z;
	nihei.x = ox*cosang-oy*sinang;
	nihei.z = oy*cosang+ox*sinang;
	ox = nifor.x; oy = nifor.z;
	nifor.x = ox*cosang-oy*sinang;
	nifor.z = oy*cosang+ox*sinang;

	nzmin = zdim/sqrt(gdd.x*gdd.x+gdd.y*gdd.y+zdim*zdim);

	for(i=numstars-1,vp=&starv[i];i>=0;i--,vp--)
	{
		nz = vp->x*nifor.x + vp->y*nifor.y + vp->z*nifor.z; if (((signed long *)&nz)[1] < ((signed long *)&nzmin)[1]) continue;
		nx = vp->x*nistr.x + vp->y*nistr.y + vp->z*nistr.z;
		ny = vp->x*nihei.x + vp->y*nihei.y + vp->z*nihei.z;

		f = halfzdim/nz;
		dtol(ny*f + halfydim,&sy); if ((unsigned)sy >= (unsigned)gdd.y) continue;
		dtol(nx*f + halfxdim,&sx); if ((unsigned)sx >= (unsigned)gdd.x) continue;
		scr = (char *)(gdd.f+ylookup[sy]+sx);
		col[0] = starcol[i]>>2; //*(sx-fsx+.5)*(sy-fsy+.5)*.25;
		//col[1] = starcol[i]*(fsx-sx+.5)*(sy-fsy+.5)*.25;
		//col[2] = starcol[i]*(sx-fsx+.5)*(fsy-sy+.5)*.25;
		//col[3] = starcol[i]*(fsx-sx+.5)*(fsy-sy+.5)*.25;
		if (scr[0] < 64) scr[0] = min(scr[0]+col[0],63);
		//if ((sx < gdd.x-1) && (scr[1] < 64)) scr[1] = min(scr[1]+col[1],63);
		//if (sy < gdd.y-1)
		//{
		//   if (scr[ylookup[1]] < 64) scr[ylookup[1]] = min(scr[ylookup[1]]+col[2],63);
		//   if ((sx < gdd.x-1) && (scr[ylookup[1]+1] < 64)) scr[ylookup[1]+1] = min(scr[ylookup[1]+1]+col[3],63);
		//}
	}


		//DRAW PLANETS

		//First calculate all 9 planet positions (and sun)
		//(need to know Earth's position first before doing other calculations)
	for(i=9;i>=0;i--)
	{
		getplanpos(i,&nx,&ny,&nz); pi[i] = i;
		px[i] = nx*nistr.x + ny*nistr.y + nz*nistr.z;
		py[i] = nx*nihei.x + ny*nihei.y + nz*nihei.z;
		pz[i] = nx*nifor.x + ny*nifor.y + nz*nifor.z;
	}

	for(i=1;i<10;i++)  //Sort out planets by distance from Earth
		for(j=0;j<i;j++)
		{
			ax = px[pi[i]]-px[2]; ay = py[pi[i]]-py[2]; az = pz[pi[i]]-pz[2];
			bx = px[pi[j]]-px[2]; by = py[pi[j]]-py[2]; bz = pz[pi[j]]-pz[2];
			if (ax*ax+ay*ay+az*az < bx*bx+by*by+bz*bz)
				{ k = pi[i]; pi[i] = pi[j]; pi[j] = k; }
		}

	for(j=9;j>=0;j--)
	{
		i = pi[j];
		if (i == 2)
		{
			getmoonpos(&ox,&oy,&oz);
			nx = ox*nistr.x + oy*nistr.y + oz*nistr.z;
			ny = ox*nihei.x + oy*nihei.y + oz*nihei.z;
			nz = ox*nifor.x + oy*nifor.y + oz*nifor.z;
		}
		else
		{
			nx = px[i]-px[2];
			ny = py[i]-py[2];
			nz = pz[i]-pz[2];
		}

		if (nz >= 0)
		{
			f = halfzdim/nz;
			dtol(nx*f + halfxdim,&sx);
			dtol(ny*f + halfydim,&sy);
			sz = 1;
		}
		else sz = -1;

		t = 1/sqrt(nx*nx+ny*ny+nz*nz);
		cx = nx*t; cy = ny*t; cz = nz*t;
		r = plandia[i]*t*0.0000005;
		//if ((i == 2) || (i == 9) || (zdim >= gdd.x*4)) //Draw ellipse if SUN,MOON or zoomed in
		if (r*zdim > gdd.x*.001) //Draw ellipse only if larger than like 1 pixel
		{
				//Get phase direction vector of planet
			if (i == 9) { kx = -cx; ky = -cy; kz = -cz; }
			else { kx = px[9]-px[i]; ky = py[9]-py[i]; kz = pz[9]-pz[i]; }
			t = 1/sqrt(kx*kx+ky*ky+kz*kz); kx *= t; ky *= t; kz *= t;

				//3D Sphere projection (see spherast.txt for derivation) (13 multiplies)
			cxcx = cx*cx; cycy = cy*cy; g = r*r - cxcx - cycy - cz*cz;
			f = halfzdim*cz - halfxdim*cx - halfydim*cy;
			a = g + cxcx;
			b = cx*cy; b += b;
			c = g + cycy;
			d = cx*f - halfxdim*g; d += d;
			e = cy*f - halfydim*g; e += e;
			f = f*f + g*halfdimsq;

			drawconic(a,b,c,d,e,f,-cx,-cy,-cz,&miny,&maxy,0);  //Only works for ellipses :(

			if (maxy >= 0)
			{
					//3D Circle projection, Input: radius:r, center:cx,cy,cz, unit_normal:kx,ky,kz  (39 *'s)
				cxcx = cx*cx; cycy = cy*cy; czcz = cz*cz;
				rkx = r*kx; rky = r*ky; rkz = r*kz;
				nx = cy*kz - cz*ky; ny = cz*kx - cx*kz; nz = cx*ky - cy*kx;
				a = nx*nx + rkx*rkx - cycy - czcz;
				b = nx*ny + rkx*rky + cx*cy; b += b;
				c = ny*ny + rky*rky - cxcx - czcz;
				d = (nx*nz + rkx*rkz + cx*cz)*halfzdim - a*halfxdim; d += d - b*halfydim;
				e = (ny*nz + rky*rkz + cy*cz)*halfzdim - c*halfydim; e += e - b*halfxdim;
				f = (nz*nz + rkz*rkz - cxcx - cycy)*halfzdim*halfzdim - a*halfxdim*halfxdim - b*halfxdim*halfydim - c*halfydim*halfydim - d*halfxdim - e*halfydim;
				drawconic(a,b,c,d,e,f,-cx,-cy,-cz,&nminy,&nmaxy,2);

				morecol = (cx*kx + cy*ky + cz*kz < 0);
				morecol = plancol[i]-((morecol^1)<<3);
				morecol = morecol+(morecol<<8)+(morecol<<16)+(morecol<<24);

				nsq = nx*nx + ny*ny + nz*nz; r2 = r*r;
				if (r2 >= nsq) //if (morecol == 0) PRINT "New/Night";  ELSE PRINT "Full/Day";
					for(y=miny;y<maxy;y++)
					{
						x1 = max(xpptr[y][0],0);
						x2 = min(xpptr[y][1],gdd.x);
						if (x1 < x2) clearbufbyte(gdd.f+ylookup[y]+x1,x2-x1,morecol);
					}
				else //if (morecol == 0) PRINT "Crescent";  ELSE PRINT "Gibbous";
				{     //Phase line projection, Input: r, c(xyz), k(xyz)
						//Output side test: (testx*ty < testy*tx+tof)
					ux = (ny*kz - nz*ky)*r2 + cx*nsq;
					uy = (nz*kx - nx*kz)*r2 + cy*nsq;
					uz = (nx*ky - ny*kx)*r2 + cz*nsq;
					tx = nz*ux - nx*uz; ty = nz*uy - ny*uz;
					tof = (halfzdim*ux + halfxdim*uz)*ty - (halfzdim*uy + halfydim*uz)*tx;
					tx *= uz; ty *= uz;
						//Hack: projects (kx,ky,kz) onto screen space to check line direction
					if ((kx*halfzdim + kz*halfxdim)*ty > (ky*halfzdim + kz*halfydim)*tx + tof*kz)
						{ tx = -tx; ty = -ty; tof = -tof; }

					y = miny;
					if (nmaxy >= 0)
					{
						usecol = (((double)((xpptr[miny][0]+xpptr[miny][1])>>1))*ty < ((double)miny)*tx+tof);
						usecol = plancol[i]-((usecol^1)<<3); usecol = usecol+(usecol<<8)+(usecol<<16)+(usecol<<24);
						while (y < nminy)
						{
							x1 = max(xpptr[y][0],0); x2 = min(xpptr[y][1],gdd.x);
							if (x1 < x2) clearbufbyte(gdd.f+ylookup[y]+x1,x2-x1,usecol);
							y++;
						}

						t = nminy*tx + tof;
						while (y < nmaxy)
						{
							x1 = max(xpptr[y][0],0); x2 = min(xpptr[y][2],gdd.x);
							if (x1 < x2)
							{
								usecol = (((double)x1)*ty < t);
								usecol = plancol[i]-((usecol^1)<<3); usecol = usecol+(usecol<<8)+(usecol<<16)+(usecol<<24);
								clearbufbyte(gdd.f+ylookup[y]+x1,x2-x1,usecol);
							}

							x2 = max(x2,0); x1 = min(xpptr[y][3],gdd.x);
							if (x2 < x1) clearbufbyte(gdd.f+ylookup[y]+x2,x1-x2,morecol);

							x1 = max(x1,0); x2 = min(xpptr[y][1],gdd.x);
							if (x1 < x2)
							{
								usecol = (((double)x2)*ty < t);
								usecol = plancol[i]-((usecol^1)<<3); usecol = usecol+(usecol<<8)+(usecol<<16)+(usecol<<24);
								clearbufbyte(gdd.f+ylookup[y]+x1,x2-x1,usecol);
							}
							t += tx; y++;
						}
					}
					usecol = (((double)((xpptr[maxy-1][0]+xpptr[maxy-1][1])>>1))*ty < ((double)(maxy-1))*tx+tof);
					usecol = plancol[i]-((usecol^1)<<3); usecol = usecol+(usecol<<8)+(usecol<<16)+(usecol<<24);
					while (y < maxy)
					{
						x1 = max(xpptr[y][0],0); x2 = min(xpptr[y][1],gdd.x);
						if (x1 < x2) clearbufbyte(gdd.f+ylookup[y]+x1,x2-x1,usecol);
						y++;
					}
				}
			}

			/*for(y=miny;y<maxy;y++)
			{
				x1 = max(xpptr[y][0],0);
				x2 = min(xpptr[y][1],gdd.x);
				if (x2 <= x1) continue;

				scr = (char *)(gdd.f+ylookup[y]);
				if (scr[x1] < 64)
				{
					if (scr[x2-1] < 64)  //Fill entire line
						clearbufbyte((long)&scr[x1],x2-x1,(long)plancol[i]+((long)plancol[i]<<8)+((long)plancol[i]<<16)+((long)plancol[i]<<24));
					else   //Unsure
					{
						for(x=x1;x<x2;x++)
							if (scr[x] < 63) scr[x] = plancol[i];
					}
				}
				else if (scr[x2-1] < 64)  //Unsure
				{
					for(x=x1;x<x2;x++)
						if (scr[x] < 63) scr[x] = plancol[i];
				}
			}*/
		}
		else   //Otherwise draw 3*3 dot
		{
			if (sz > 0)
				for(y=-1;y<=1;y++)
				{
					if ((unsigned)(sy+y) >= (unsigned)gdd.y) continue;
					for(x=-1;x<=1;x++)
					{
						if ((unsigned)(sx+x) >= (unsigned)gdd.x) continue;
						scr = (char *)(gdd.f+ylookup[sy+y]+sx+x);
						if (*scr < (plancol[i]&63)) *scr = plancol[i];
					}
				}
		}
		if ((planamode) && (sz > 0))
		{
			x = sx-strlen(planame[i])*3; y = sy-10;
			print6x8(x,y,31,-1,"%s",(char *)planame[i]);
		}
	}
}

static void drawsnow ()
{
	double dax, day, daz, ox, oy, oz, nx, ny, nz;
	long i, sx, sy;

	dax = fmod(ipos.x*16,16384); if (dax < 0) dax += 16384;
	day = fmod(ipos.y*16,16384); if (day < 0) day += 16384;
	daz = fmod(ipos.z*16,16384); if (daz < 0) daz += 16384;
	for(i=0;i<MAXSNOWS;i++)
	{
		ox = snowpos[i].x-dax; if (ox < 0) ox += 16384;
		oy = snowpos[i].y-day; if (oy < 0) oy += 16384;
		oz = snowpos[i].z-daz; if (oz < 0) oz += 16384;
		ox -= 8192;
		oy -= 8192;
		oz -= 8192;
		if (ox*ox + oy*oy + oz*oz >= 8192*8192) continue;
		nz = ox*ifor.x + oy*ifor.y + oz*ifor.z; if (nz <= 0) continue;
		nx = ox*istr.x + oy*istr.y + oz*istr.z;
		ny = ox*ihei.x + oy*ihei.y + oz*ihei.z;

		dtol(ny/nz*halfzdim + halfydim,&sy);
		if ((unsigned long)sy < (unsigned long)gdd.y)
		{
			dtol(nx/nz*halfzdim + halfxdim,&sx);
			if ((unsigned long)sx < (unsigned long)gdd.x)
			{
				if (!(rand()&15)) *(char *)(gdd.f+ylookup[sy]+sx) = (rand()&255);
								 else *(char *)(gdd.f+ylookup[sy]+sx) = 63-(nz*(1.0/128));
			}
		}
	}
}

static long globeinside (double x, double y, double z)
{
#if (USEGLOBEINSIDE != 0)
	double flat, flng, x1, y1, x2, y2, d;
	long i, j, cnt;

	flng = atan2(x,-z)*fhlonunits/PI;
	flat = atan2(y,sqrt(x*x+z*z))*fhlonunits/PI;

		//Maximum lng delta is maxdeltlon, (maximum lat delta is 215*?)
	cnt = 0;

		//Ugly case - when flng is close to International Date Line
	if (fabs(flng) >= fhlonunits-maxdatelon*1.2)  //1.2 for good luck
	{
		if (flng < 0) flng += fhlonunits; else flng -= fhlonunits;
		for(i=0;i<numpoints;i++)
		{
			j = index[i];
			if (ll[j].lng < 0) y1 = ((double)ll[j].lng)+fhlonunits-flng;
							  else y1 = ((double)ll[j].lng)-fhlonunits-flng;
			if (y1 > maxdatelon) break;
			if (ll[point2[j]].lng < 0) y2 = ((double)ll[point2[j]].lng)+fhlonunits-flng;
										 else y2 = ((double)ll[point2[j]].lng)-fhlonunits-flng;
			if ((y1 < 0) == (y2 < 0)) continue;
			x1 = ((double)ll[j].lat)-flat;
			x2 = ((double)ll[point2[j]].lat)-flat;
			if ((x1 < 0) == (x2 < 0)) { if (x1 < 0) cnt ^= 1; continue; }
			if ((x1*y2 < x2*y1) != (y2 < 0)) cnt ^= 1;
		}
		for(i=numpoints-1;i>=0;i--)
		{
			j = index[i];
			if (ll[j].lng < 0) y1 = ((double)ll[j].lng)+fhlonunits-flng;
							  else y1 = ((double)ll[j].lng)-fhlonunits-flng;
			if (y1 < -maxdatelon) break;
			if (ll[point2[j]].lng < 0) y2 = ((double)ll[point2[j]].lng)+fhlonunits-flng;
										 else y2 = ((double)ll[point2[j]].lng)-fhlonunits-flng;
			if ((y1 < 0) == (y2 < 0)) continue;
			x1 = ((double)ll[j].lat)-flat;
			x2 = ((double)ll[point2[j]].lat)-flat;
			if ((x1 < 0) == (x2 < 0)) { if (x1 < 0) cnt ^= 1; continue; }
			if ((x1*y2 < x2*y1) != (y2 < 0)) cnt ^= 1;
		}
		return(cnt);
	}

		//Evil double float compare tricks (need to convert above code also!)
	i = 0;
	for(j=numpoints>>1;j>0;j>>=1)
		{ d = flng-ll[index[i+j]].lng; if (((long *)&d)[1] > ((long *)&maxdeltlon)[1]) i += j; }
	for(;i<numpoints;i++)
	{
		j = index[i];
		y1 = ((double)ll[j].lng)-flng; if (((long *)&y1)[1] > ((long *)&maxdeltlon)[1]) break;
		y2 = ((double)ll[point2[j]].lng)-flng;
		if ((((long *)&y1)[1] ^ ((long *)&y2)[1]) >= 0) continue;
		x1 = ((double)ll[j].lat)-flat;
		x2 = ((double)ll[point2[j]].lat)-flat;
		if ((((long *)&x1)[1] ^ ((long *)&x2)[1]) >= 0)
			{ if (((long *)&x1)[1] < 0) cnt ^= 1; continue; }
		d = x1*y2 - x2*y1; if ((((long *)&d)[1] ^ ((long *)&y2)[1]) < 0) cnt ^= 1;
	}
	return(cnt);
#else
	dpoint3d p0, p1, p2, p3;
	double n, d, mind, dot, bestd;
	long i, j, k, l, m, bestk, mini[3], ispt, minispt;

	d = 1.0/sqrt(x*x+y*y+z*z); x *= d; y *= d; z *= d;

	mind = 1e32; mini[0] = 0; mini[1] = 1; mini[2] = 2;

	//1. find mdist: dist of lps with minimum maxdist
	//2. process all lps with minimum dist < mdist

	bestd = -1e32; bestk = 0;
	for(k=lpstotal-1;k>=0;k--)
	{
		dot = lps[k].x*x + lps[k].y*y + lps[k].z*z;
		d = lps[k].c*dot - lps[k].s*sqrt(1-dot*dot);
		if (d > bestd) { bestd = d; bestk = k; }
	}
		//Refine max dist to check using quick approximation
	if (!bestk) i = 0; else i = lps[bestk-1].np;
	for(;i<lps[bestk].np;i+=16)
	{
		d = v[i].x*x + v[i].y*y + v[i].z*z;
		if (d > bestd) bestd = d;
	}

	for(k=lpstotal-1;k>=0;k--)
	{
		dot = lps[k].x*x + lps[k].y*y + lps[k].z*z;
		d = lps[k].c*dot + lps[k].s*sqrt(1-dot*dot);
		if ((dot < lps[k].c) && (d < bestd)) continue;

		i = lps[k].np-1;
		if (!k) m = 0; else m = lps[k-1].np;
		l = lps[k].nl;
		for(;i>=m;i--)
		{
			while (i < loopoff[l]) { l--; if (l < 0) break; }

			j = i+1; if (j >= loopoff[l+1]) j = loopoff[l];
			p0.x = v[i].x-x; p1.x = v[j].x-v[i].x;
			p0.y = v[i].y-y; p1.y = v[j].y-v[i].y;
			p0.z = v[i].z-z; p1.z = v[j].z-v[i].z;
			n = -(p0.x*p1.x + p0.y*p1.y + p0.z*p1.z);
			ispt = 1;
			if (((long *)&n)[1] > 0)
			{
				d = p1.x*p1.x + p1.y*p1.y + p1.z*p1.z;
				if (n < d)
				{
					d = n/d;
					p0.x += p1.x*d;
					p0.y += p1.y*d;
					p0.z += p1.z*d;
					ispt = 0;
				}
			}
			d = p0.x*p0.x + p0.y*p0.y + p0.z*p0.z;
			if (d < mind)
			{
				mind = d;
				mini[0] = i-1; if (mini[0] < loopoff[l]) mini[0] = loopoff[l+1]-1;
				mini[1] = i;
				mini[2] = j;
				minispt = ispt;
#ifdef DBGNEARLIN
				dbgnearlin[2].x = p0.x+x; dbgnearlin[2].y = p0.y+y; dbgnearlin[2].z = p0.z+z;
#endif
			}
		}
	}

	p0.x = v[mini[0]].x; p0.y = v[mini[0]].y; p0.z = v[mini[0]].z;
	p1.x = v[mini[1]].x; p1.y = v[mini[1]].y; p1.z = v[mini[1]].z;
	p2.x = v[mini[2]].x; p2.y = v[mini[2]].y; p2.z = v[mini[2]].z;
	if (!minispt) { p0 = p1; p1 = p2; }
	else
	{
			//Make sure line isn't blocked ... check point side of line twice
		p3.x = p1.y*z - p1.z*y;
		p3.y = p1.z*x - p1.x*z;
		p3.z = p1.x*y - p1.y*x;
		if ((p0.x*p3.x + p0.y*p3.y + p0.z*p3.z < 0.0) == (p2.x*p3.x + p2.y*p3.y + p2.z*p3.z < 0.0))
		{
			p3.x = p1.y*p2.z - p1.z*p2.y;
			p3.y = p1.z*p2.x - p1.x*p2.z;
			p3.z = p1.x*p2.y - p1.y*p2.x;
			if ((p0.x*p3.x + p0.y*p3.y + p0.z*p3.z < 0.0) != (x*p3.x + y*p3.y + z*p3.z < 0.0))
				{ p0 = p1; p1 = p2; }
		}
	}
#ifdef DBGNEARLIN
	dbgnearlin[0].x = p0.x; dbgnearlin[0].y = p0.y; dbgnearlin[0].z = p0.z;
	dbgnearlin[1].x = p1.x; dbgnearlin[1].y = p1.y; dbgnearlin[1].z = p1.z;
#endif

	p3.x = p0.y*p1.z - p0.z*p1.y;
	p3.y = p0.z*p1.x - p0.x*p1.z;
	p3.z = p0.x*p1.y - p0.y*p1.x;
	return(x*p3.x + y*p3.y + z*p3.z >= 0.0);

#endif
}

static long klgif (char *filename)
{
	long i, j, x, y, xspan, yspan, currstr, numbitgoal, fil;
	long lzcols, palcols, dat, blocklen, p, pend, bitcnt;
	char numbits, startnumbits, chunkind, *ptr, interlaced;

	if (!strstr(filename,".")) strcat(filename,".gif");
	if ((fil = open(filename,O_BINARY|O_RDONLY,S_IREAD)) == -1) return(-1);
	read(fil,gifdata,GIFBUFLEN);
	if ((gifdata[0] != 'G') || (gifdata[1] != 'I') ||
		 (gifdata[2] != 'F') || (gifdata[12])) return(-1);
	palcols = (1<<((gifdata[10]&7)+1));
	ptr = &gifdata[13];
	if (gifdata[10]&128) { memcpy(filbuffer,ptr,palcols*3); ptr += palcols*3; }
	while ((chunkind = *ptr++) == '!')
	{
		ptr++;
		do { i = *ptr++; ptr += i; } while (i);
	}
	if (chunkind != ',') { close(fil); return(-1); }

	xspan = *(short *)&ptr[4]; yspan = *(short *)&ptr[6]; ptr += 9;
	if (ptr[-1]&64) interlaced = 8+16; else interlaced = 0;
	if (ptr[-1]&128)
	{
		palcols = (1<<((ptr[-1]&7)+1));
		memcpy(filbuffer,ptr,palcols*3); ptr += palcols*3;
	}

	for(i=0;i<palcols;i++)
	{
		pal[i].peRed = filbuffer[i*3+0];
		pal[i].peGreen = filbuffer[i*3+1];
		pal[i].peBlue = filbuffer[i*3+2];
	}

	yspan = min(yspan,xspan>>1); //Chop off extra y to force 2:1 ratio

	picxdim = xspan; picydim = yspan;
	if (!(pic = (char *)malloc(picxdim*picydim))) MessageBox(0,"Out of memory (pic)",0,MB_OK);

	p = 0; pend = xspan; y = 0;

	lzcols = (1<<(*ptr)); startnumbits = (*ptr)+1; ptr++;
	for(i=lzcols-1;i>=0;i--) { suffix[i] = prefix[i] = i; }
	currstr = lzcols+2; numbits = startnumbits; numbitgoal = (lzcols<<1);
	blocklen = *ptr++;
	memcpy(filbuffer,ptr,blocklen); ptr += blocklen;
	bitcnt = 0;
	while (1)
	{
		dat = ((*(long *)&filbuffer[bitcnt>>3])>>(bitcnt&7)) & (numbitgoal-1);
		bitcnt += numbits;
		if ((bitcnt>>3) > blocklen-3)
		{
			*(short *)filbuffer = *(short *)&filbuffer[bitcnt>>3];
			i = blocklen-(bitcnt>>3);
			blocklen = (long)*ptr++;
			j = GIFBUFLEN-(ptr-gifdata);
			if (blocklen < j)
				{ memcpy(&filbuffer[i],ptr,blocklen); ptr += blocklen; }
			else
			{
				memcpy(&filbuffer[i],ptr,j);
				read(fil,gifdata,GIFBUFLEN);
				memcpy(&filbuffer[i+j],gifdata,blocklen-j);
				ptr = &gifdata[blocklen-j];
			}
			bitcnt &= 7; blocklen += i;
		}
		if (dat == lzcols)
		{
			currstr = lzcols+2; numbits = startnumbits; numbitgoal = (lzcols<<1);
			continue;
		}
		if ((currstr == numbitgoal) && (numbits < 12))
			{ numbits++; numbitgoal <<= 1; }

		prefix[currstr] = dat;
		for(i=0;dat>=lzcols;dat=prefix[dat]) tempstack[i++] = suffix[dat];
		tempstack[i] = prefix[dat];
		suffix[currstr-1] = suffix[currstr] = dat;
		for(;i>=0;i--)
		{
			pic[p++] = tempstack[i];
			if (p >= pend)
			{
				if (!interlaced)
				{
					y++;
					if (y >= yspan) { close(fil); return(0); }
				}
				else
				{
					y += (interlaced&15);
					if (y >= yspan)
					{
						y = 0;
						switch(interlaced)
						{
							case 24: interlaced = 8; y += 4; break;
							case 8: interlaced = 4; y += 2; break;
							case 4: interlaced = 2; y++; break;
							case 2: close(fil);
							return(0);
						}
						p = y*xspan;
					}
					else
						p += ((interlaced&15)-1)*xspan;
				}
				pend = p+xspan;
			}
		}
		currstr++;
	}
}

static void drawtextureglobe ()
{
	double f, fx, fy, ox, oy, oz, rx, ry, rz, Za, Zb, Zc, isqr;
	long x, y, x1, x2, lat, lng, picoffs;
	char *ptr;

	fx = (double)(picxdim)/(PI+PI);
	fy = (double)(picydim)/PI;
	picoffs = (picydim+1)*(picxdim>>1);
	for(y=0;y<gdd.y;y++)
	{
		if (xpcnt[y] != 2)
			{ clearbufbyte(gdd.f+ylookup[y],gdd.x,0L); continue; }
		x1 = kbound0val(xpptr[y][0],gdd.x);
		x2 = kbound0val(xpptr[y][1],gdd.x);
		clearbufbyte(gdd.f+ylookup[y],x1,0L);
		clearbufbyte(gdd.f+ylookup[y]+x2,gdd.x-x2,0L);

		//clearbufbyte(gdd.f+ylookup[y]+x1,x2-x1,0x35353535);

		rx = (x1-halfxdim)*istr.x + (y-halfydim)*ihei.x + halfzdim*ifor.x;
		ry = (x1-halfxdim)*istr.y + (y-halfydim)*ihei.y + halfzdim*ifor.y;
		rz = (x1-halfxdim)*istr.z + (y-halfydim)*ihei.z + halfzdim*ifor.z;

		ptr = (char *)(gdd.f+ylookup[y]+x1);
		Zc = ipos.x*ipos.x + ipos.y*ipos.y + ipos.z*ipos.z - 1;
		for(x=x1;x<x2;x++)
		{
			Za = rx*rx + ry*ry + rz*rz;
			Zb = ipos.x*rx + ipos.y*ry + ipos.z*rz;
			isqr = Zb*Zb - Za*Zc;
			if (isqr < 0)
			{
				*ptr++ = 0;
			}
			else
			{
				f = (-Zb - sqrt(isqr)) / Za;
				lng = (long)(atan2(ipos.x+f*rx,-ipos.z-f*rz)*fx);
				lat = (long)(asin(min(max(ipos.y+f*ry,-1),1))*fy);
				*ptr++ = pic[lat*picxdim+lng+picoffs];
			}
			rx += istr.x; ry += istr.y; rz += istr.z;
		}
	}
}

static void drawconic (double a, double b, double c, double d, double e, double f,
			 double cx, double cy, double cz, long *miny, long *maxy, long ind)
{
	double g, h, i, s, si, sii, x1, x2, sxm, sxmi;
	long sy;

	(*miny) = 0.0; (*maxy) = -1.0;

		//Equation of valid half of conic section: cx*sx + cy*sy + i < 0
	if (a == 0) return;
	h = -.5 / a; i = halfxdim*cx + halfydim*cy - halfzdim*cz;
	sxm = d*h; sxmi = b*h;
	s = d*d - a*f*4; si = b*(b+d+d) - a*(c+e)*4; sii = b*b*2 - a*c*8;
	for(sy=0;sy<gdd.y;sy++) //(a)x + (by+d)x + (cy+ey+f) = 0
	{
		if (s > 0)
		{
			g = sqrt(s)*h; x1 = sxm-g; x2 = sxm+g;
			if (x1*cx < i)
			{
				if (x2*cx < i)
				{
					if ((x1 < gdd.x) && (x2 > 0))
					{
						if (x1 < 0) xpptr[sy][ind] = 0; else dtol(x1,&xpptr[sy][ind]);
						if (x2 > gdd.x) xpptr[sy][ind+1] = gdd.x; else dtol(x2,&xpptr[sy][ind+1]);
						if ((*maxy) < 0) (*miny) = sy;
						(*maxy) = sy;
					}
				}
				else
				{
					if (x1 < gdd.x)
					{
						if (x1 < 0) xpptr[sy][ind] = 0; else dtol(x1,&xpptr[sy][ind]);
						xpptr[sy][ind+1] = gdd.x;
						if ((*maxy) < 0) (*miny) = sy;
						(*maxy) = sy;
					}
				}
			}
			else if (x2*cx < i)
			{
				if (x2 > 0)
				{
					xpptr[sy][ind] = 0;
					if (x2 > gdd.x) xpptr[sy][ind+1] = gdd.x; else dtol(x2,&xpptr[sy][ind+1]);
					if ((*maxy) < 0) (*miny) = sy;
					(*maxy) = sy;
				}
			}
		}
		s += si; si += sii;
		i -= cy; sxm += sxmi;
	}
	if ((*maxy) >= 0) (*maxy)++; else { *miny = 0; }

#if 0
	{
	double Za, Zb, Zc;
	Za = b*b - a*c*4.0;
	Zb = b*d - a*e*2.0;
	Zc = d*d - a*f*4.0;
	Zc = Zb*Zb - Za*Zc;
	if ((Zc >= 0) && (Za != 0))
	{
		Zc = sqrt(Zc); if (Za < 0) Zc = -Zc;
		Za = 1.0/Za;
		g = (-Zb-Zc)*Za; if (g > (*miny)) (*miny) = g;
		g = (-Zb+Zc)*Za; if (g < (*maxy)) (*maxy) = g;
		if ((*miny) >= (*maxy)) { (*miny) = 0; (*maxy) = -1; }
	}
	}
#endif
}

static void drawbluesphere ()
{
	double a, b, c, d, e, f, cx, cy, cz;
	long i, miny, maxy;

	cx = ipos.x*istr.x + ipos.y*istr.y + ipos.z*istr.z;
	cy = ipos.x*ihei.x + ipos.y*ihei.y + ipos.z*ihei.z;
	cz = ipos.x*ifor.x + ipos.y*ifor.y + ipos.z*ifor.z;

		//3D Sphere projection (see spherast.txt for derivation) (13 multiplies)
	f = halfzdim*cz - halfxdim*cx - halfydim*cy;
	a = cx*cx; c = cy*cy; b = 1 - a - c - cz*cz;
	a += b; c += b;
	d = cx*f - halfxdim*b; d += d;
	e = cy*f - halfydim*b; e += e;
	f = f*f + b*halfdimsq;
	b = cx*cy; b += b;

	drawconic(a,b,c,d,e,f,cx,cy,cz,&miny,&maxy,0);
	if (maxy >= 0)
	{
		clearbufbyte((long)&xpcnt[   0],(miny -   0)<<2,0L); for(i=   0;i< miny;i++) xpptr[i][0] = 0;
		clearbufbyte((long)&xpcnt[miny],(maxy -miny)<<2,2L);
		clearbufbyte((long)&xpcnt[maxy],(gdd.y-maxy)<<2,0L); for(i=maxy;i<gdd.y;i++) xpptr[i][0] = 0;
	}
	else
	{
		clearbufbyte((long)xpcnt,gdd.y<<2,0L); for(i=0;i<gdd.y;i++) xpptr[i][0] = 0;
	}
}

static double minrad;
static char landwaterfilename[256], polbordsfilename[256];
long initapp (long argc, char **argv)
{
	double r, ox, oy, oz;
	long i, j, k, fil, gotres;
	short *sptr;
	char ch;

	prognam = "Globe by Ken Silverman (http://advsys.net/ken)";
	colbits = 8; fullscreen = 1;
	strcpy(landwaterfilename,"SHOREX.DAT");
	strcpy(polbordsfilename,"BORDX.DAT");
	gotres = 0;
	for(i=argc-1;i>0;i--)
	{
		if ((argv[i][0] == '/') || (argv[i][0] == '-'))
		{
			if ((argv[i][1] >= '0') && (argv[i][1] <= '9'))
			{
				k = 0; ch = 0;
				for(j=1;;j++)
				{
					if ((argv[i][j] >= '0') && (argv[i][j] <= '9'))
						{ k = (k*10+argv[i][j]-48); continue; }
					switch (ch)
					{
						case 0: xres = k; break;
						case 1: yres = k; break;
						//case 2: cols = k; break;
					}
					if (!argv[i][j]) break;
					ch++; if (ch > 1) break;  //(ch > 2)
					k = 0;
				}
				gotres = 1;
			}
			else if ((argv[i][1] == 'W') || (argv[i][1] == 'w')) { fullscreen = 0; }
			else if ((argv[i][1] == 'X') || (argv[i][1] == 'x')) { startlng = atof(&argv[i][2]); }
			else if ((argv[i][1] == 'Y') || (argv[i][1] == 'y')) { startlat = atof(&argv[i][2]); }
		}
		else
		{
			if (((argv[i][0] == 'S') || (argv[i][0] == 's')) && ((argv[i][1] == 'H') || (argv[i][1] == 'h')))
			{
				strcpy(landwaterfilename,argv[i]);
				if (!strchr(landwaterfilename,'.'))
					strcat(landwaterfilename,".dat");
			}
			if (((argv[i][0] == 'B') || (argv[i][0] == 'b')) && ((argv[i][1] == 'O') || (argv[i][1] == 'o')))
			{
				strcpy(polbordsfilename,argv[i]);
				if (!strchr(polbordsfilename,'.'))
					strcat(polbordsfilename,".dat");
			}
		}
	}

	if (!gotres)
	{
		if (fullscreen)
		{
			HDC hDC = GetDC(0);
			xres = GetDeviceCaps(hDC,HORZRES);
			yres = GetDeviceCaps(hDC,VERTRES);
			ReleaseDC(0,hDC);
		}
		else
		{
			RECT rw;
			SystemParametersInfo(SPI_GETWORKAREA,0,&rw,0);
			xres = rw.right -rw.left;
			yres = rw.bottom-rw.top - GetSystemMetrics(SM_CYCAPTION);
		}
	}

	for(i=0;i<64;i++) { pal[i].peRed = (i<<2); pal[i].peGreen = (i<<2); pal[i].peBlue = (i<<2); }           //Stars/sky
	for(i=1;i<=16;i++)
	{
		pal[i+0x3f].peRed = ((i*63)>>2); pal[i+0x3f].peGreen = ((i*48)>>2); pal[i+0x3f].peBlue = ((i*28)>>2); //Land
		pal[i+0x4f].peRed = ((i*16)>>2); pal[i+0x4f].peGreen = ((i*32)>>2); pal[i+0x4f].peBlue = ((i*63)>>2); //Water
		pal[i+0x5f].peRed = ((i*63)>>2); pal[i+0x5f].peGreen = ((i*60)>>2); pal[i+0x5f].peBlue = ((i*56)>>2); //Mercury
		pal[i+0x6f].peRed = ((i*63)>>2); pal[i+0x6f].peGreen = ((i*52)>>2); pal[i+0x6f].peBlue = ((i*20)>>2); //Venus
		pal[i+0x7f].peRed = ((i*63)>>2); pal[i+0x7f].peGreen = ((i*63)>>2); pal[i+0x7f].peBlue = ((i*56)>>2); //Moon
		pal[i+0x8f].peRed = ((i*63)>>2); pal[i+0x8f].peGreen = ((i*28)>>2); pal[i+0x8f].peBlue = ((i*16)>>2); //Mars
		pal[i+0x9f].peRed = ((i*63)>>2); pal[i+0x9f].peGreen = ((i*48)>>2); pal[i+0x9f].peBlue = ((i*28)>>2); //Jupiter
		pal[i+0xaf].peRed = ((i*56)>>2); pal[i+0xaf].peGreen = ((i*48)>>2); pal[i+0xaf].peBlue = ((i*24)>>2); //Saturn
		pal[i+0xbf].peRed = ((i* 8)>>2); pal[i+0xbf].peGreen = ((i*40)>>2); pal[i+0xbf].peBlue = ((i*32)>>2); //Uranus
		pal[i+0xcf].peRed = ((i* 8)>>2); pal[i+0xcf].peGreen = ((i*24)>>2); pal[i+0xcf].peBlue = ((i*44)>>2); //Neptune
		pal[i+0xdf].peRed = ((i* 8)>>2); pal[i+0xdf].peGreen = ((i*24)>>2); pal[i+0xdf].peBlue = ((i*16)>>2); //Pluto
		pal[i+0xef].peRed = ((i*63)>>2); pal[i+0xef].peGreen = ((i*63)>>2); pal[i+0xef].peBlue = ((i*16)>>2); //Sun
	}

	for(i=0;i<MAXSNOWS;i++)
	{
		snowpos[i].x = (float)(rand()&16383);
		snowpos[i].y = (float)(rand()&16383);
		snowpos[i].z = (float)(rand()&16383);
	}

	texturemode = (klgif("globe.gif") >= 0);

	loadshore(landwaterfilename);
	loadpolbords(polbordsfilename);

	if ((fil = open("stars.dat",O_BINARY|O_RDONLY)) == -1)
	{
		MessageBox(0,"Can't find STARS.DAT\nStars disabled...\n",0,MB_OK);
		numstars = 0;
	}
	else
	{
		read(fil,&numstars,4);
		if (starv != 0) free(starv);
		if (!(starv = (fpoint3d *)malloc(numstars*sizeof(fpoint3d)))) MessageBox(0,"Out of memory (starv)",0,MB_OK);
		if (starcol != 0) free(starcol);
		if (!(starcol = (char *)malloc(numstars))) MessageBox(0,"Out of memory (starcol)",0,MB_OK);
			//Notice that starv is the wrong type!  This is a HACK to avoid a malloc!
		read(fil,starv,numstars*4);
		read(fil,starcol,numstars);
		close(fil);
		sptr = (short *)starv;
		for(i=numstars-1;i>=0;i--)
		{
			dcossin(((double)sptr[i*2+1])*(PI/65536.0),&r,&oy);
			dcossin(((double)sptr[i*2])*(PI/32768.0),&oz,&ox);
			starv[i].x = ox*r; starv[i].y = oy; starv[i].z = oz*-r;
		}
	}
	totalclock = ototalclock = lockclock = timeGetTime();
	synctics = 0;

		//TIME_ZONE_INFORMATION structure members:
		//LONG Bias;
		//WCHAR StandardName[32]; SYSTEMTIME StandardDate; LONG StandardBias;
		//WCHAR DaylightName[32]; SYSTEMTIME DaylightDate; LONG DaylightBias;
	GetTimeZoneInformation(&tzinfo);

	minrad = (MINMILESABOVESURFACE+earthdia*.5)/(earthdia*.5);
	return(0);
}

void uninitapp ()
{
	if (xp) { free(xp); xp = 0; }
	if (starcol) free(starcol);
	if (starv) free(starv);
	if (polv) free(polv);
	if (polineto) free(polineto);
#if (USEGLOBEINSIDE != 0)
	if (index) { free(index); index = 0; }
	if (point2) { free(point2); point2 = 0; }
	if (ll) { free(ll); ll = 0; }
#else
	if (ybit) { free(ybit); ybit = 0; }
	if (zbit) { free(zbit); zbit = 0; }
#endif
	if (loopoff) { free(loopoff); loopoff = 0; }
	if (lps) { free(lps); lps = 0; }
	if (v) { free(v); v = 0; }
	if (pic) { free(pic); pic = 0; }
}

static long capturecount = 0;
static char tempbuf[4096];
static char pcxheader[128] =
{
	10,5,1,8,0,0,0,0,63,1,199,0,64,1,200,0,
	0,0,0,8,8,8,16,16,16,24,24,24,32,32,32,40,40,40,48,48,48,56,56,56,64,64,64,
	72,72,72,80,80,80,88,88,88,96,96,96,104,104,104,112,112,112,120,120,120,
	0,1,64,1,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
	0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0
};

static long screencapture8bit (char *filename, char *frameptr, long dabpl, long daxdim, long daydim, PALETTEENTRY *dapal)
{
	long fil, x, i, j, p, col, ncol, leng, pend;

	if ((fil=open(filename,O_BINARY|O_CREAT|O_TRUNC|O_WRONLY,S_IWRITE)) == -1) return(-1);
	*(short *)(&pcxheader[8]) = (short)(daxdim-1);
	*(short *)(&pcxheader[10]) = (short)(daydim-1);
	*(short *)(&pcxheader[12]) = *(short *)(&pcxheader[66]) = (short)daxdim;
	*(short *)(&pcxheader[14]) = (short)daydim;
	write(fil,pcxheader,128);

	i = 0; x = p = 0; pend = daxdim*daydim;
	while (p < pend)
	{
		col = *frameptr; p++; x++; if (x >= daxdim) { x = 0; frameptr += dabpl-daxdim; }
		frameptr++; leng = 1; ncol = *frameptr;
		while ((ncol == col) && (p < pend) && (leng < 63) && ((p%daxdim) != 0))
		{
			leng++; p++; x++; if (x >= daxdim) { x = 0; frameptr += dabpl-daxdim; }
			frameptr++; ncol = *frameptr;
		}
		if ((leng > 1) || (col >= 0xc0)) tempbuf[i++] = (leng|0xc0);
		tempbuf[i++] = col;
		if (i > sizeof(tempbuf)-2)
		{
			if (write(fil,tempbuf,i) < i) { close(fil); return(-1); }
			i = 0;
		}
	}

	if (i > sizeof(tempbuf)-768-1)
	{
		if (write(fil,tempbuf,i) < i) { close(fil); return(-1); }
		i = 0;
	}

	tempbuf[i++] = 0xc;
	for(j=0;j<256;j++)
	{
		tempbuf[i++] = pal[j].peRed;
		tempbuf[i++] = pal[j].peGreen;
		tempbuf[i++] = pal[j].peBlue;
	}

	if (write(fil,tempbuf,i) < i) { close(fil); return(-1); }

	close(fil);
	return(0);
}

void doframe ()
{
	double ox, oy, oz, nx, ny, nz, a1, a2, a3, r, rr[9];
	double cosa1, sina1, cosa2, sina2, cosa3, sina3, c1c3, c1s3, s1c3, s1s3;
	double px[10], py[10], pz[10];
	long i, j, k, z, zz, zx, t, fil, startz;
	long x, y, x1, y1, x2, y2, pag;
	long dabytesperline, daxdim, daydim;
	char tbuf[256];
	wchar_t *wptr;

	totalclock = timeGetTime();
	readkeyboard();
	obstatus = bstatus; readmouse(&fmousx,&fmousy,&bstatus);

	if (!(bstatus&2))
	{
		a1 = 0; if (gdd.x != -1) a3 = fmousx*gdd.x/(zdim*128); else a3 = 0;
		if ((drawearth) && (tiltfixmode))  //Tiltfix code derived from ab = cos()ab
		{
			nx = ipos.x*ipos.x + ipos.y*ipos.y + ipos.z*ipos.z;
			if (nx < 4*4)
			{
				ny = istr.x*istr.x + istr.y*istr.y + istr.z*istr.z;
				nz = ipos.x*istr.x + ipos.y*istr.y + ipos.z*istr.z;
				a1 = nz/(nx*ny*(-8));
			}
		}
	}
	else { a1 = fmousx/(-128); a3 = 0; }
	if (gdd.x != -1) a2 = fmousy*gdd.x/(zdim*(-128)); else a2 = 0;

	if (ext_keystatus[0x29]) { ext_keystatus[0x29] = 0; planamode ^= 1; }

	if (keystatus[0xc]|keystatus[0xd])
	{
		if (keystatus[0x38]|keystatus[0xb8])
		{
			gday += keystatus[0xd]-keystatus[0xc];
			keystatus[0xc] = keystatus[0xd] = 0;
		}
		else
		{
			gsecond = 0;
			if (keystatus[0x2a]) //Single step
			{
				gminute += (keystatus[0xd]-keystatus[0xc]);
				keystatus[0xc] = keystatus[0xd] = 0;
			}
			else
				gminute += (keystatus[0xd]-keystatus[0xc])*2;
		}
		angmessed = 1;
		initdatetime(0L);
	}
	if (keystatus[0x1a]|keystatus[0x1b]) //[]
	{
		if (keystatus[0x38]|keystatus[0xb8])
		{
			if (keystatus[0x2a]|keystatus[0x36])
				gyear += (keystatus[0x1b]-keystatus[0x1a])*10;
			else
				gyear += keystatus[0x1b]-keystatus[0x1a];
			keystatus[0x1a] = keystatus[0x1b] = 0;
		}
		else
		{
			j = isdaylight(gyear,gmonth,gday,ghour);
			i = getday(gyear,gmonth,gday);
			getdate(i+keystatus[0x1b]-keystatus[0x1a],&gyear,&gmonth,&gday);
			if (j != isdaylight(gyear,gmonth,gday,ghour))
				{ if (j) ghour--; else ghour++; }
			if (keystatus[0x36])  //23:56 instead of 24:00 increment
			{
					//Earth sidereal mean:       23:56:04.091
				gminute -= 4*(keystatus[0x1b]-keystatus[0x1a]);
				gsecond += 4*(keystatus[0x1b]-keystatus[0x1a]);
			}
				//Single step
			if (keystatus[0x2a]) keystatus[0x1a] = keystatus[0x1b] = 0;
		}
		angmessed = 1;
		initdatetime(0L);
	}
	if (keystatus[0x33]|keystatus[0x34])
	{
		dcossin(((double)((keystatus[0x34]-keystatus[0x33])*(keystatus[0x2a]+1)*(keystatus[0x36]*3+1)))/128.0,&cosa1,&sina1);
		ox = ipos.x; oy = ipos.z;
		ipos.x = ox*cosa1-oy*sina1;
		ipos.z = oy*cosa1+ox*sina1;
		ox = istr.x; oy = istr.z;
		istr.x = ox*cosa1-oy*sina1;
		istr.z = oy*cosa1+ox*sina1;
		ox = ihei.x; oy = ihei.z;
		ihei.x = ox*cosa1-oy*sina1;
		ihei.z = oy*cosa1+ox*sina1;
		ox = ifor.x; oy = ifor.z;
		ifor.x = ox*cosa1-oy*sina1;
		ifor.z = oy*cosa1+ox*sina1;
	}
	if (keystatus[0xb5]|keystatus[0x37])  //Keyp. /, Keyp. *
	{
			//Setting keystatus[?] = 0 doesn't work for 0xb5
			//because keyp. / stupidly repeats when held
		if ((!zoompause) && (gdd.x != -1))
		{
			r = zdim;
			zdim *= (1+(keystatus[0x37]-keystatus[0xb5])*.02);

				//Stop at 90
			if ((r < gdd.x) && (zdim >= gdd.x)) { zdim = gdd.x; zoompause = 1; }
			if ((r > gdd.x) && (zdim <= gdd.x)) { zdim = gdd.x; zoompause = 1; }

			halfzdim = zdim*.5;
			halfdimsq = halfxdim*halfxdim + halfydim*halfydim + halfzdim*halfzdim;
		}
	} else zoompause = 0;

	if ((keystatus[0x4c]) && (gdd.x != -1))  //Keyp. 5 - reset zoom to 1
	{
		keystatus[0x4c] = 0;
		zdim = gdd.x; zoompause = 1;

		halfzdim = zdim*.5;
		halfdimsq = halfxdim*halfxdim + halfydim*halfydim + halfzdim*halfzdim;
	}

	if (!angmessed)
		initdatetime(1L);

	dcossin(a1,&cosa1,&sina1);
	dcossin(a2,&cosa2,&sina2);
	dcossin(a3,&cosa3,&sina3);
	c1c3 = cosa1*cosa3; c1s3 = cosa1*sina3;
	s1c3 = sina1*cosa3; s1s3 = sina1*sina3;
	rr[0] = s1s3*sina2 + c1c3; rr[1] = -c1s3*sina2 + s1c3; rr[2] = sina3*cosa2;
	rr[3] = -cosa2*sina1;      rr[4] = cosa2*cosa1;   rr[5] = sina2;
	rr[6] = s1c3*sina2 - c1s3; rr[7] = -c1c3*sina2 - s1s3; rr[8] = cosa3*cosa2;
	ox = istr.x; oy = ihei.x; oz = ifor.x;
	istr.x = ox*rr[0] + oy*rr[3] + oz*rr[6];
	ihei.x = ox*rr[1] + oy*rr[4] + oz*rr[7];
	ifor.x = ox*rr[2] + oy*rr[5] + oz*rr[8];
	ox = istr.y; oy = ihei.y; oz = ifor.y;
	istr.y = ox*rr[0] + oy*rr[3] + oz*rr[6];
	ihei.y = ox*rr[1] + oy*rr[4] + oz*rr[7];
	ifor.y = ox*rr[2] + oy*rr[5] + oz*rr[8];
	ox = istr.z; oy = ihei.z; oz = ifor.z;
	istr.z = ox*rr[0] + oy*rr[3] + oz*rr[6];
	ihei.z = ox*rr[1] + oy*rr[4] + oz*rr[7];
	ifor.z = ox*rr[2] + oy*rr[5] + oz*rr[8];

	synctics = totalclock-lockclock; lockclock += synctics;

	r = (double)synctics/2048.0;
	if (keystatus[0x2a]) r = (double)synctics/512.0;
	if (keystatus[0x36])
	{
		r = (double)synctics/64.0;
		if ((ivel.x*ivel.x + ivel.y*ivel.y + ivel.z*ivel.z)*earthdia*earthdia*.5*.5*1000*1000*.25*.25 > SPEEDOFLIGHT*SPEEDOFLIGHT)
			r = sqrt(ivel.x*ivel.x + ivel.y*ivel.y + ivel.z*ivel.z)*1.1;
	}
	//r = (double)synctics/2048.0;
	//if (keystatus[0x2a]) r = (double)synctics/512.0;
	//if (keystatus[0x36]) r = (double)synctics/64.0;

	if (!gravity)
	{
		if (keystatus[0xcb]) { ipos.x -= istr.x*r; ipos.y -= istr.y*r; ipos.z -= istr.z*r; }
		if (keystatus[0xcd]) { ipos.x += istr.x*r; ipos.y += istr.y*r; ipos.z += istr.z*r; }
		if (keystatus[0x9d]) { ipos.x -= ihei.x*r; ipos.y -= ihei.y*r; ipos.z -= ihei.z*r; }
		if (keystatus[0x52]) { ipos.x += ihei.x*r; ipos.y += ihei.y*r; ipos.z += ihei.z*r; }
		if (keystatus[0xc8]) { ipos.x += ifor.x*r; ipos.y += ifor.y*r; ipos.z += ifor.z*r; }
		if (keystatus[0xd0]) { ipos.x -= ifor.x*r; ipos.y -= ifor.y*r; ipos.z -= ifor.z*r; }
	}
	else
	{
		r *= .01;
		if (keystatus[0xcb]) { ivel.x -= istr.x*r; ivel.y -= istr.y*r; ivel.z -= istr.z*r; }
		if (keystatus[0xcd]) { ivel.x += istr.x*r; ivel.y += istr.y*r; ivel.z += istr.z*r; }
		if (keystatus[0x9d]) { ivel.x -= ihei.x*r; ivel.y -= ihei.y*r; ivel.z -= ihei.z*r; }
		if (keystatus[0x52]) { ivel.x += ihei.x*r; ivel.y += ihei.y*r; ivel.z += ihei.z*r; }
		if (keystatus[0xc8]) { ivel.x += ifor.x*r; ivel.y += ifor.y*r; ivel.z += ifor.z*r; }
		if (keystatus[0xd0]) { ivel.x -= ifor.x*r; ivel.y -= ifor.y*r; ivel.z -= ifor.z*r; }
	}

	if (keystatus[0x35]) { keystatus[0x35] = 0; initposatkenshouse(); } // / (forward slash-reset)
	if (keystatus[0x13]) //R (randomize)
	{
		keystatus[0x13] = 0;

			//UNIFORM spherical randomization (see spherand.c)
		ipos.z = ((double)(rand()&32767))/16383.5-1.0;
		dcossin((((double)(rand()&32767))/16383.5-1.0)*PI,&ipos.x,&ipos.y);
		r = sqrt(1.0 - ipos.z*ipos.z); ipos.x *= r; ipos.y *= r;

		ifor.x = -ipos.x; ifor.y = -ipos.y; ifor.z = -ipos.z;

			//UNIFORM spherical randomization (see spherand.c)
		istr.z = ((double)(rand()&32767))/16383.5-1.0;
		dcossin((((double)(rand()&32767))/16383.5-1.0)*PI,&istr.x,&istr.y);
		r = sqrt(1.0 - istr.z*istr.z); istr.x *= r; istr.y *= r;

			//Graham-Schmidt (suck ifor out of istr)
		r = -(istr.x*ifor.x + istr.y*ifor.y + istr.z*ifor.z);
		istr.x += ifor.x*r; istr.y += ifor.y*r; istr.z += ifor.z*r;

			//Normalize istr
		r = 1 / sqrt(istr.x*istr.x + istr.y*istr.y + istr.z*istr.z);
		istr.x *= r; istr.y *= r; istr.z *= r;

			//Use cross product to get 3rd vector
		ihei.x = ifor.y*istr.z - ifor.z*istr.y;
		ihei.y = ifor.z*istr.x - ifor.x*istr.z;
		ihei.z = ifor.x*istr.y - ifor.y*istr.x;
	}
	if (keystatus[0x26])  //L
	{
		keystatus[0x26] = 0;
		loadshore(landwaterfilename);
		loadpolbords(polbordsfilename);
	}
	/*if ((keystatus[0x1f]) && (keystatus[0x38]))  //L.Alt-S
	{
		keystatus[0x1f] = 0;
		if ((fil = open("?.dat",O_BINARY|O_TRUNC|O_CREAT|O_WRONLY,S_IWRITE)) == -1)
			MessageBox(0,"Can't save globe.dat\n",0,MB_OK);
		else
		{
			write(fil,?,?);  //Note: Update for new format! (July, 1999)
			close(fil);
		}
	}*/
	if (ext_keystatus[0x23]) { ext_keystatus[0x23] = 0; drawearth ^= 1; } //H-hide earth
	if (ext_keystatus[0x14]) { ext_keystatus[0x14] = 0; transparent ^= 1; } //T
	if (ext_keystatus[0x30]) { ext_keystatus[0x30] = 0; polborders ^= 1; } //B
	if (ext_keystatus[0x1e]) { ext_keystatus[0x1e] = 0; tiltfixmode ^= 1; } //A
	if (ext_keystatus[0x22]) { ext_keystatus[0x22] = 0; gravity ^= 1; ivel.x = ivel.y = ivel.z = 0; } //G
	if (ext_keystatus[0x39]) { ext_keystatus[0x39] = 0; ivel.x = ivel.y = ivel.z = 0; } //Space

	if (gravity)
	{
		r = ipos.x*ipos.x + ipos.y*ipos.y + ipos.z*ipos.z;
		r = -.0003 / (r*sqrt(r));
		//r = (-.00000000153584865/(1000*.25)) / (r*sqrt(r));  //True gravity constant (BORING!!!)
		ivel.x += ipos.x*r*((double)synctics/16);
		ivel.y += ipos.y*r*((double)synctics/16);
		ivel.z += ipos.z*r*((double)synctics/16);
		ipos.x += ivel.x*((double)synctics/16);
		ipos.y += ivel.y*((double)synctics/16);
		ipos.z += ivel.z*((double)synctics/16);

			//F = G*m1*m2 / r
			//G = 6.673*10^-11 (m^3)/(kg*s)
			//Mearth = 5,975,510,000,000,000,000,000 kg
			//a = 6.673*10^-11 * 5.97551*10^21 / (r*r)
			//1 Earth-rad = 6,379,439.616 meters
			//a = 398,745,700,000 / (r*r) (r, a in meters,sec)
			//a = 398,745,700,000 / (r*r) (r, a in Earth-rads,sec)

			//Speed of light is 299792.458km/s or 186282mil/s rad.Earth=3964mil
			//Speed of light is: 186282/3964 units/sec (about 23.5 earths/s)

			//Make sure you don't go faster than speed of light
		//r = sqrt(ivel.x*ivel.x + ivel.y*ivel.y + ivel.z*ivel.z);
		//if (r > (SPEEDOFLIGHT/(earthdia*.5)) / (1000*.25))
		//{
		//   r = ((SPEEDOFLIGHT/(earthdia*.5)) / (1000*.25)) / r;
		//   ivel.x *= r; ivel.y *= r; ivel.z *= r;
		//}
	}

	r = sqrt(ipos.x*ipos.x + ipos.y*ipos.y + ipos.z*ipos.z);
	if (r < minrad)
	{
		r = minrad / r;
		ipos.x *= r; ipos.y *= r; ipos.z *= r;
		r = pow(.99,synctics); ivel.x *= r; ivel.y *= r; ivel.z *= r;
	}

	if (keystatus[0x2e]) facecenter();//C (face center, preserve angle)
	if (keystatus[0x20]) facedown();  //D (face down)
	if (keystatus[0x2c]) faceup();    //Z (zenith key)
	if (keystatus[0x31]) facenorth(); //N (face north)
	if (keystatus[0x1f]) facesouth(); //S (face south)
	if (keystatus[0x12]) faceeast();  //E (face east)
	if (keystatus[0x11]) facewest();  //W (face west)

	for(i=9;i>=0;i--)
		if (keystatus[i+2])
		{
			if (i == 2)
				getmoonpos(&nx,&ny,&nz);
			else
			{
				getplanpos(i,&nx,&ny,&nz);
				getplanpos(2L,&px[2],&py[2],&pz[2]);
				nx -= px[2]; ny -= py[2]; nz -= pz[2];
			}
			dcossin(starang,&cosa1,&sina1);
			ox = nx*cosa1+nz*sina1; oy = ny; oz = nz*cosa1-nx*sina1;

			if (keystatus[0x2b]) //Backslash
			{
				keystatus[0x2b] = 0;
				if (!(keystatus[0x2a]|keystatus[0x36]))
				{
					j = 0;  //Find closest sunrise/sunset/transit

					rr[0] = ox*ipos.x + oy*ipos.y + oz*ipos.z;
					rr[1] = ox*-ipos.z + oy*0 + oz*ipos.x;
					rr[2] = -(ox*ipos.x + oy*ipos.y + oz*ipos.z);
					rr[3] = -(ox*-ipos.z + oy*0 + oz*ipos.x);
					if ((fabs(rr[0]) <= fabs(rr[1])) && (fabs(rr[0]) <= fabs(rr[2])) && (fabs(rr[0]) <= fabs(rr[3])))
						k = 0^((rr[1] < 0)<<1);
					if ((fabs(rr[1]) <= fabs(rr[0])) && (fabs(rr[1]) <= fabs(rr[2])) && (fabs(rr[1]) <= fabs(rr[3])))
						k = 1^((rr[2] < 0)<<1);
					if ((fabs(rr[2]) <= fabs(rr[0])) && (fabs(rr[2]) <= fabs(rr[1])) && (fabs(rr[2]) <= fabs(rr[3])))
						k = 2^((rr[3] < 0)<<1);
					if ((fabs(rr[3]) <= fabs(rr[0])) && (fabs(rr[3]) <= fabs(rr[1])) && (fabs(rr[3]) <= fabs(rr[2])))
						k = 3^((rr[0] < 0)<<1);

					rr[4] = 0;
					if ((i == 2) || (i == 9))
					{
						if (k == 0) rr[4] = sin(SUNMOONRISETANGLE);
						if (k == 2) rr[4] = -sin(SUNMOONRISETANGLE);
					}
					else
					{
						if (k == 0) rr[4] = sin(DOTRISETANGLE);
						if (k == 2) rr[4] = -sin(DOTRISETANGLE);
					}

					rr[0] = 0;
					while ((!keystatus[1]) && (!keystatus[0x2b]))
					{
						if (i == 2)
							getmoonpos(&nx,&ny,&nz);
						else
						{
							getplanpos(i,&nx,&ny,&nz);
							getplanpos(2L,&px[2],&py[2],&pz[2]);
							nx -= px[2]; ny -= py[2]; nz -= pz[2];
						}
						dcossin(starang,&cosa1,&sina1);
						ox = nx*cosa1+nz*sina1; oy = ny; oz = nz*cosa1-nx*sina1;

						rr[1] = rr[0];
						switch(k)
						{
							case 0: rr[0] = ox*ipos.x + oy*ipos.y + oz*ipos.z; break;
							case 1: rr[0] = ox*-ipos.z + oy*0 + oz*ipos.x; break;
							case 2: rr[0] = -(ox*ipos.x + oy*ipos.y + oz*ipos.z); break;
							case 3: rr[0] = -(ox*-ipos.z + oy*0 + oz*ipos.x); break;
						}
						if (!(k&1))
						{   //Make ipos & o unit vectors for dot product angle
							r = ipos.x*ipos.x + ipos.y*ipos.y + ipos.z*ipos.z;
							if (r > 0) rr[0] /= sqrt(r);
							r = ox*ox + oy*oy + oz*oz;
							if (r > 0) rr[0] /= sqrt(r);
						}
						if ((rr[0] <= rr[4]) && (rr[1] > rr[4]))
						{
							if (j >= 2) break;
							j++;
						}
						switch(j)
						{
							case 0: if (rr[0] < rr[4]) ghour++; else ghour--; break;
							case 1: if (rr[0] < rr[4]) gminute++; else gminute--; break;
							case 2: if (rr[0] < rr[4]) gsecond++; else gsecond--; break;
						}
						initdatetime(0L);
					}
				}
				else if (i == 9)
				{
					j = 0; //Find next closest vernal/autumnal equinox
					k = ((gmonth+1)/3)&3;  //Ex: Feb 1 to Apr 30 goes to Mar 22
					if (k&1)
					{
						rr[0] = 0;
						while ((!keystatus[1]) && (!keystatus[0x2b]))
						{
							getplanpos(2L,&nx,&ny,&nz);

							rr[1] = rr[0];
							if (k == 3) rr[0] = -ny; else rr[0] = ny;
							if ((rr[0] <= 0) && (rr[1] > 0))
							{
								if (j >= 3) break;
								j++;
							}
							switch(j)
							{
								case 0: if (rr[0] < 0) gday++; else gday--; break;
								case 1: if (rr[0] < 0) ghour++; else ghour--; break;
								case 2: if (rr[0] < 0) gminute++; else gminute--; break;
								case 3: if (rr[0] < 0) gsecond++; else gsecond--; break;
							}
							initdatetime(0L);
						}
					}
				}
				keystatus[0x2b] = 0;
				angmessed = 1;
			}

			facedir(ox,oy,oz);
			break;
		}

	if (startdirectdraw(&gdd.f,&dabytesperline,&daxdim,&daydim))
	{
		if ((gdd.p != dabytesperline) || (gdd.x != daxdim) || (gdd.y != daydim))
		{
			gdd.p = dabytesperline;
			gdd.x = daxdim;
			gdd.y = daydim;

			if (ylookup) { free(ylookup); ylookup = 0; }
			if (!(ylookup = (long *)malloc(gdd.y*sizeof(long))))
				{ MessageBox(0,"Out of memory (ylookup)",0,MB_OK); stopdirectdraw(); nextpage(); quitloop(); return; }
			ylookup[0] = 0; for(i=1;i<gdd.y;i++) ylookup[i] = ylookup[i-1]+gdd.p;
			halfxdim = (double)gdd.x * 0.5;
			halfydim = (double)gdd.y * 0.5;

				//For some reason, GLOBE screen starts black if this call isn't here
			initposatkenshouse();

			zdim = gdd.x; halfzdim = zdim*.5;
			halfdimsq = halfxdim*halfxdim + halfydim*halfydim + halfzdim*halfzdim;

			if (xpptr) { free(xpptr); xpptr = 0; }
			if (!(xpptr = (long **)malloc(gdd.y*sizeof(long *))))
				{ MessageBox(0,"Out of memory (xpptr)",0,MB_OK); stopdirectdraw(); nextpage(); quitloop(); return; }

			if (xpcnt) { free(xpcnt); xpcnt = 0; }
			if (!(xpcnt = (long *)malloc(gdd.y*sizeof(long))))
				{ MessageBox(0,"Out of memory (xpcnt)",0,MB_OK); stopdirectdraw(); nextpage(); quitloop(); return; }

			if (xp) { free(xp); xp = 0; }
			if (!(xp = (long *)malloc(gdd.y*MAXCROSSESPERLINE*sizeof(long))))
				{ MessageBox(0,"Out of memory (xp)",0,MB_OK); stopdirectdraw(); nextpage(); quitloop(); return; }
			for(i=0;i<gdd.y;i++) xpptr[i] = &xp[i*MAXCROSSESPERLINE];

#if (USEGLOBEINSIDE == 0)
			if (zbit) { free(zbit); zbit = 0; }
				//only need 3 bits per pixel... but 8 is easier
			if (!(zbit = (unsigned char *)malloc((gdd.x|1)*gdd.y)))
				{ MessageBox(0,"Out of memory (zbit)",0,MB_OK); stopdirectdraw(); nextpage(); quitloop(); return; }

			if (ybit) { free(ybit); ybit = 0; }
			if (!(ybit = (unsigned char **)malloc(gdd.y*sizeof(unsigned char *))))
				{ MessageBox(0,"Out of memory (ybit)",0,MB_OK); stopdirectdraw(); nextpage(); quitloop(); return; }

			j = (gdd.x|1); ybit[0] = zbit; for(i=1;i<gdd.y;i++) ybit[i] = ybit[i-1]+j;
#endif
		}

		if (drawearth)
		{
			drawbluesphere();
			if (texturemode)
				drawtextureglobe();
			else
				drawland();
			if (polborders) drawpolbords();
		}
		else
			clearbufbyte(gdd.f,gdd.p*gdd.y,0L);
		if (!texturemode)
			drawstars(starang);

		r = sqrt(ivel.x*ivel.x + ivel.y*ivel.y + ivel.z*ivel.z);
		if (r*earthdia*.5*1000*.25 > SPEEDOFLIGHT) drawsnow();

		print6x8(0L,0L,31,-1,"%s %ld, %ld (%s)",month[gmonth-1],gday,gyear,dayofweek[getdayofweek(getday(gyear,gmonth,gday))]);
		if (!isdaylight(gyear,gmonth,gday,ghour)) { wptr = (wchar_t *)&tzinfo.StandardName; i = tzinfo.Bias+tzinfo.StandardBias; }
														 else { wptr = (wchar_t *)&tzinfo.DaylightName; i = tzinfo.Bias+tzinfo.DaylightBias; }
		print6x8(0L,10L,31,-1,"%2ld:%02ld:%02ld %cM %ls (UT%c%02ld:%02ld)",((ghour+11)%12)+1,gminute,gsecond,'A'+(('P'-'A')&(-(ghour>=12))),wptr,'+'+(('-'-'+')&(-(i>0))),labs(i)/60,labs(i)%60);

		if (gravity)
		{
			r = 1.0 / sqrt(ipos.x*ipos.x+ipos.y*ipos.y+ipos.z*ipos.z);
			ox = ipos.x*r; oy = ipos.y*r; oz = ipos.z*r;
				//ox = cos(lat)*sin(lng);
				//oy = sin(lat);
				//oz = -cos(lat)*cos(lng);
			ny = asin(min(max(oy,-1),1))*180/PI;     //lat = asin(oy);
			nx = atan2(ox,-oz)*180.0/PI;        //lng = asin(ox/cos(lat));
			print6x8(0L,20L,31,-1,"%f %c%c",fabs(ny),248,'N'+(ny>0)*5);
			print6x8(0L,30L,31,-1,"%f %c%c",fabs(nx),248,'E'+(nx<0)*18);

			r = sqrt(ivel.x*ivel.x + ivel.y*ivel.y + ivel.z*ivel.z);
			print6x8(0L,gdd.y-8,31,-1,"%f%%",100*r/((SPEEDOFLIGHT/(earthdia*.5))/(1000*.25)));
		}

		if (gdd.x != zdim)
		{
			r = atan(gdd.x/zdim)*360/PI;
			print6x8(0L,40L,31,-1,"Magnification: %f, Viewing range: %ld%c %ld\' %ld\"",zdim/gdd.x,(long)r,248,(long)(r*60)%60,(long)(r*3600)%60);
		}

		if (ext_keystatus[0x58])  //F12
		{
			ext_keystatus[0x58] = 0;
			strcpy(tbuf,"GLOB0000.PCX");
			do
			{
				tbuf[4] = ((capturecount/1000)%10)+48;
				tbuf[5] = ((capturecount/100)%10)+48;
				tbuf[6] = ((capturecount/10)%10)+48;
				tbuf[7] = (capturecount%10)+48;
				i = open(tbuf,O_BINARY|O_RDONLY,S_IREAD); if (i == -1) break;
				close(i); capturecount++;
			} while (1);
			if (screencapture8bit(tbuf,(char *)gdd.f,gdd.p,gdd.x,gdd.y,pal) == 0)
				capturecount++;
		}
		stopdirectdraw();
		nextpage();
	}

	if (keystatus[1]) quitloop();
}

#if 0
!endif
#endif
